from typing import Final, Literal

AF_12844: Final[int]
AF_APPLETALK: Final[int]
AF_ASH: Final[int]
AF_ATM: Final[int]
AF_ATMPVC: Final[int]
AF_ATMSVC: Final[int]
AF_AX25: Final[int]
AF_BAN: Final[int]
AF_BLUETOOTH: Final[int]
AF_BRIDGE: Final[int]
AF_DATAKIT: Final[int]
AF_DECnet: Final[int]
AF_CCITT: Final[int]
AF_CHAOS: Final[int]
AF_CLUSTER: Final[int]
AF_CNT: Final[int]
AF_COIP: Final[int]
AF_DLI: Final[int]
AF_ECONET: Final[int]
AF_ECMA: Final[int]
AF_FILE: Final[int]
AF_FIREFOX: Final[int]
AF_HYLINK: Final[int]
AF_IMPLINK: Final[int]
AF_INET: Final[int]
AF_INET6: Final[int]
AF_IPX: Final[int]
AF_IRDA: Final[int]
AF_ISDN: Final[int]
AF_ISO: Final[int]
AF_KEY: Final[int]
AF_LAT: Final[int]
AF_LINK: Final[int]
AF_NATM: Final[int]
AF_NETBEUI: Final[int]
AF_NETBIOS: Final[int]
AF_NETDES: Final[int]
AF_NETGRAPH: Final[int]
AF_NETLINK: Final[int]
AF_NETROM: Final[int]
AF_NDRV: Final[int]
AF_NS: Final[int]
AF_PACKET: Final[int]
AF_PPP: Final[int]
AF_PPPOX: Final[int]
AF_PUP: Final[int]
AF_ROSE: Final[int]
AF_ROUTE: Final[int]
AF_SECURITY: Final[int]
AF_SIP: Final[int]
AF_SNA: Final[int]
AF_SYSTEM: Final[int]
AF_UNIX: Final[int]
AF_UNKNOWN1: Final[int]
AF_UNSPEC: Final[int]
AF_VOICEVIEW: Final[int]
AF_WANPIPE: Final[int]
AF_X25: Final[int]
IN6_IFF_AUTOCONF: Final[int]
IN6_IFF_TEMPORARY: Final[int]
IN6_IFF_DYNAMIC: Final[int]
IN6_IFF_OPTIMISTIC: Final[int]
IN6_IFF_SECURED: Final[int]

address_families: Final[dict[int, str]]
version: Final[str]

def gateways() -> dict[int | Literal["default"], list[tuple[str, str, bool] | tuple[str, str]] | dict[int, tuple[str, str]]]: ...
def ifaddresses(ifname: str, /) -> dict[int, list[dict[str, str]]]: ...
def interfaces() -> list[str]: ...



from typing import Literal

from .enums import Duplex, PageBoundaries, PageMode, TextDirection

class ViewerPreferences:
    hide_toolbar: bool
    hide_menubar: bool
    hide_window_u_i: bool
    fit_window: bool
    center_window: bool
    def __init__(
        self,
        hide_toolbar: bool = False,
        hide_menubar: bool = False,
        hide_window_u_i: bool = False,
        fit_window: bool = False,
        center_window: bool = False,
        display_doc_title: bool = False,
        non_full_screen_page_mode: PageMode | str | None = ...,
        num_copies: int | None = None,
        print_page_range: list[int] | None = None,
        direction: TextDirection | str | None = None,
        duplex: Duplex | str | None = None,
        view_area: PageBoundaries | None = None,
        view_clip: PageBoundaries | None = None,
        print_area: PageBoundaries | None = None,
        print_clip: PageBoundaries | None = None,
        print_scaling=None,
    ) -> None: ...
    @property
    def non_full_screen_page_mode(self) -> PageMode | None: ...
    @non_full_screen_page_mode.setter
    def non_full_screen_page_mode(self, page_mode: PageMode | str | None) -> None: ...
    @property
    def num_copies(self) -> int | None: ...
    @num_copies.setter
    def num_copies(self, num_copies: int | None) -> None: ...
    @property
    def print_page_range(self) -> list[int] | None: ...
    @print_page_range.setter
    def print_page_range(self, print_page_range: list[int] | None) -> None: ...
    @property
    def direction(self) -> TextDirection | None: ...
    @direction.setter
    def direction(self, direction: TextDirection | str | None) -> None: ...
    @property
    def display_doc_title(self) -> bool: ...
    @display_doc_title.setter
    def display_doc_title(self, display_doc_title: bool) -> None: ...
    @property
    def duplex(self) -> Duplex | None: ...
    @duplex.setter
    def duplex(self, duplex: Duplex | str | None) -> None: ...
    @property
    def view_area(self) -> PageBoundaries | None: ...
    @view_area.setter
    def view_area(self, view_area: PageBoundaries | str | None) -> None: ...
    @property
    def view_clip(self) -> PageBoundaries | None: ...
    @view_clip.setter
    def view_clip(self, view_area: PageBoundaries | str | None) -> None: ...
    @property
    def print_area(self) -> PageBoundaries | None: ...
    @print_area.setter
    def print_area(self, view_area: PageBoundaries | str | None) -> None: ...
    @property
    def print_clip(self) -> PageBoundaries | None: ...
    @print_clip.setter
    def print_clip(self, view_area: PageBoundaries | str | None) -> None: ...
    @property
    def print_scaling(self) -> Literal["None", "AppDefault"] | None: ...
    @print_scaling.setter
    def print_scaling(self, print_scaling: Literal["None", "AppDefault"] | None) -> None: ...
    def serialize(self) -> str: ...



from _typeshed import Incomplete
from collections.abc import Mapping

from braintree.exchange_rate_quote import ExchangeRateQuote

class ExchangeRateQuotePayload:
    quotes: list[ExchangeRateQuote]
    def __init__(self, data: Mapping[str, Incomplete]) -> None: ...
    def get_quotes(self) -> list[ExchangeRateQuote]: ...



from _typeshed import Incomplete, SupportsLenAndGetItem
from collections.abc import Callable, Iterable, Mapping
from typing import Any, Literal, TypeVar

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "traveling_salesman_problem",
    "christofides",
    "asadpour_atsp",
    "greedy_tsp",
    "simulated_annealing_tsp",
    "threshold_accepting_tsp",
]

_SupportsLenAndGetItemT = TypeVar("_SupportsLenAndGetItemT", bound=SupportsLenAndGetItem[Any])

def swap_two_nodes(soln: _SupportsLenAndGetItemT, seed) -> _SupportsLenAndGetItemT: ...
def move_one_node(soln: _SupportsLenAndGetItemT, seed) -> _SupportsLenAndGetItemT: ...
@_dispatchable
def christofides(G: Graph[_Node], weight: str | None = "weight", tree: Graph[_Node] | None = None) -> list[Incomplete]: ...
@_dispatchable
def traveling_salesman_problem(
    G: Graph[_Node],
    weight: str = "weight",
    nodes=None,
    cycle: bool = True,
    method: Callable[..., Incomplete] | None = None,
    **kwargs,
): ...
@_dispatchable
def asadpour_atsp(
    G: DiGraph[_Node], weight: str | None = "weight", seed: int | RandomState | None = None, source: str | None = None
): ...
@_dispatchable
def held_karp_ascent(G: Graph[_Node], weight="weight"): ...
@_dispatchable
def spanning_tree_distribution(G: Graph[_Node], z: Mapping[Incomplete, Incomplete]) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def greedy_tsp(G: Graph[_Node], weight: str | None = "weight", source=None): ...
@_dispatchable
def simulated_annealing_tsp(
    G: Graph[_Node],
    init_cycle,
    weight: str | None = "weight",
    source=None,
    temp: int | None = 100,
    move="1-1",
    max_iterations: int | None = 10,
    N_inner: int | None = 100,
    alpha=0.01,
    seed: int | RandomState | None = None,
): ...
@_dispatchable
def threshold_accepting_tsp(
    G: Graph[_Node],
    init_cycle: Literal["greedy"] | Iterable[Incomplete],
    weight: str | None = "weight",
    source=None,
    threshold: int | None = 1,
    move="1-1",
    max_iterations: int | None = 10,
    N_inner: int | None = 100,
    alpha=0.1,
    seed: int | RandomState | None = None,
): ...



from collections.abc import Iterator
from typing import Generic, TypeVar
from typing_extensions import Self

_T_co = TypeVar("_T_co", covariant=True)

class CancellableStream(Generic[_T_co]):
    def __init__(self, stream: Iterator[_T_co], response) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...
    next = __next__
    def close(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Sequence

from networkx.utils.backends import _dispatchable

__all__ = ["combinatorial_embedding_to_pos"]

@_dispatchable
def combinatorial_embedding_to_pos(embedding, fully_triangulate: bool = False) -> dict[Incomplete, Incomplete]: ...
def set_position(parent, tree, remaining_nodes, delta_x, y_coordinate, pos): ...
def get_canonical_ordering(embedding, outer_face: Sequence[Incomplete]) -> list[Incomplete]: ...
def triangulate_face(embedding, v1, v2): ...
def triangulate_embedding(embedding, fully_triangulate: bool = True): ...
def make_bi_connected(
    embedding, starting_node, outgoing_node, edges_counted: set[tuple[Incomplete, Incomplete]]
) -> list[Incomplete]: ...



from _typeshed import Unused
from collections.abc import Sequence
from typing import Final

from Xlib.display import Display
from Xlib.protocol import rq
from Xlib.xobject import resource

RES_MAJOR_VERSION: Final = 1
RES_MINOR_VERSION: Final = 2
extname: Final = "X-Resource"
ResQueryVersion: Final = 0
ResQueryClients: Final = 1
ResQueryClientResources: Final = 2
ResQueryClientPixmapBytes: Final = 3
ResQueryClientIds: Final = 4
ResQueryResourceBytes: Final = 5

class QueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource, client_major: int = 1, client_minor: int = 2) -> QueryVersion: ...

Client: rq.Struct

class QueryClients(rq.ReplyRequest): ...

def query_clients(self: Display | resource.Resource) -> QueryClients: ...

Type: rq.Struct

class QueryClientResources(rq.ReplyRequest): ...

def query_client_resources(self: Display | resource.Resource, client: int) -> QueryClientResources: ...

class QueryClientPixmapBytes(rq.ReplyRequest): ...

def query_client_pixmap_bytes(self: Display | resource.Resource, client: int) -> QueryClientPixmapBytes: ...

class SizeOf(rq.LengthOf):
    item_size: int
    def __init__(self, name: str | list[str] | tuple[str, ...], size: int, item_size: int) -> None: ...
    def parse_value(self, length: int, display: Unused) -> int: ...  # type: ignore[override]

ClientXIDMask: Final = 0x1
LocalClientPIDMask: Final = 0x2
ClientIdSpec: rq.Struct
ClientIdValue: rq.Struct

class QueryClientIds(rq.ReplyRequest): ...

def query_client_ids(self: Display | resource.Resource, specs: Sequence[tuple[int, int]]) -> QueryClientIds: ...

ResourceIdSpec: rq.Struct
ResourceSizeSpec: rq.Struct
ResourceSizeValue: rq.Struct

class QueryResourceBytes(rq.ReplyRequest): ...

def query_resource_bytes(
    self: Display | resource.Resource, client: int, specs: Sequence[tuple[int, int]]
) -> QueryResourceBytes: ...
def init(disp: Display, info: Unused) -> None: ...



from ..extractor.common import _InfoDict
from .fragment import FragmentFD

class HlsFD(FragmentFD):
    FD_NAME: str
    @classmethod
    def can_download(cls, manifest: str, info_dict: _InfoDict, allow_unplayable_formats: bool = False) -> bool: ...



from _typeshed.wsgi import WSGIEnvironment
from collections.abc import Awaitable, Callable
from typing import TypedDict, type_check_only

@type_check_only
class _SocketDataBase(TypedDict):
    type: str

@type_check_only
class _SocketDataProtocol(_SocketDataBase):
    subprotocol: str | None

@type_check_only
class _SocketDataStr(_SocketDataBase):
    text: str

@type_check_only
class _SocketDataBytes(_SocketDataBase):
    bytes: bytes

class WebSocketASGI:
    subprotocols: list[str]
    subprotocol: str
    connected: bool
    # this is set in `close` to `False`
    conncted: bool
    def __init__(
        self,
        scope: WSGIEnvironment,
        receive: Callable[[], Awaitable[_SocketDataBytes | _SocketDataStr]],
        send: Callable[[_SocketDataBase | _SocketDataProtocol | _SocketDataBytes | _SocketDataStr], Awaitable[None]],
        subprotocols: list[str] | None = None,
    ) -> None: ...
    @classmethod
    async def accept(
        cls,
        scope: WSGIEnvironment,
        receive: Callable[[], Awaitable[_SocketDataBytes | _SocketDataStr]],
        send: Callable[[_SocketDataBase | _SocketDataProtocol | _SocketDataBytes | _SocketDataStr], Awaitable[None]],
        subprotocols: list[str] | None = None,
    ) -> WebSocketASGI: ...
    async def receive(self) -> bytes | str: ...
    async def send(self, data: bytes | str) -> None: ...
    async def close(self) -> None: ...



from collections.abc import Sequence

from docker.context.context import Context
from docker.tls import TLSConfig

class ContextAPI:
    DEFAULT_CONTEXT: Context
    @classmethod
    def create_context(
        cls,
        name: str,
        orchestrator: str | None = None,
        host: str | None = None,
        tls_cfg: TLSConfig | None = None,
        default_namespace: str | None = None,
        skip_tls_verify: bool = False,
    ) -> Context: ...
    @classmethod
    def get_context(cls, name: str | None = None) -> Context: ...
    @classmethod
    def contexts(cls) -> Sequence[Context]: ...
    @classmethod
    def get_current_context(cls) -> Context: ...
    @classmethod
    def set_current_context(cls, name: str = "default") -> None: ...
    @classmethod
    def remove_context(cls, name: str) -> None: ...
    @classmethod
    def inspect_context(cls, name: str = "default") -> Context: ...



from collections.abc import Iterable, Mapping
from decimal import Decimal
from re import Pattern
from typing import Any, Literal, TypedDict, type_check_only
from typing_extensions import NotRequired

from .adjacency_graphs import _Graph

@type_check_only
class _Match(TypedDict):
    pattern: Literal["dictionary", "spatial", "repeat", "sequence", "regex", "date"]
    token: str
    i: int
    j: int
    guesses: NotRequired[int]  # all patterns except 'date'
    guesses_log10: NotRequired[float]  # all patterns except 'date'

    # pattern == 'date'
    separator: NotRequired[str]
    year: NotRequired[int]
    month: NotRequired[int]
    day: NotRequired[int]

    # pattern == 'dictionary'
    matched_word: NotRequired[str]
    dictionary_name: NotRequired[str]
    l33t: NotRequired[bool]
    reversed: NotRequired[bool]
    rank: NotRequired[int]
    base_guesses: NotRequired[int | Decimal]  # Decimal for 'repeat', see below
    uppercase_variations: NotRequired[int]
    l33t_variations: NotRequired[int]

    # pattern == 'spatial'
    turns: NotRequired[int]

    # pattern == 'repeat'
    base_token: NotRequired[str]
    # base_guesses: NotRequired[Decimal]
    base_matches: NotRequired[list[Any]]  # Any = _Match, https://github.com/python/mypy/issues/731
    repeat_count: NotRequired[float]

    # pattern == 'regex'
    regex_name: NotRequired[str]

def build_ranked_dict(ordered_list: Iterable[str]) -> dict[str, int]: ...

RANKED_DICTIONARIES: dict[str, dict[str, int]]

def add_frequency_lists(frequency_lists_: Mapping[str, Iterable[str]]) -> None: ...

GRAPHS: dict[str, dict[str, list[str | None]]]
L33T_TABLE: dict[str, list[str]]
REGEXEN: dict[str, Pattern[str]]
DATE_MAX_YEAR: int
DATE_MIN_YEAR: int
DATE_SPLITS: dict[int, list[list[int]]]

def omnimatch(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
def dictionary_match(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
def reverse_dictionary_match(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
def relevant_l33t_subtable(password: str, table: Mapping[str, Iterable[str]]) -> dict[str, list[str]]: ...
def enumerate_l33t_subs(table: Mapping[str, Iterable[str]]) -> list[dict[str, str]]: ...
def translate(string: str, chr_map: Mapping[str, str]) -> str: ...
def l33t_match(
    password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ..., _l33t_table: dict[str, list[str]] = ...
) -> list[_Match]: ...
def repeat_match(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
def spatial_match(
    password: str, _graphs: dict[str, _Graph] = ..., _ranked_dictionaries: dict[str, dict[str, int]] = ...
) -> list[_Match]: ...

SHIFTED_RX: Pattern[str]

def spatial_match_helper(password: str, graph: _Graph, graph_name: str) -> list[_Match]: ...

MAX_DELTA: int

def sequence_match(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
def regex_match(
    password: str, _regexen: dict[str, Pattern[str]] = ..., _ranked_dictionaries: dict[str, dict[str, int]] = ...
) -> list[_Match]: ...
def date_match(password: str, _ranked_dictionaries: dict[str, dict[str, int]] = ...) -> list[_Match]: ...
@type_check_only
class _DM(TypedDict):
    month: int
    day: int

@type_check_only
class _DMY(TypedDict):
    year: int
    month: int
    day: int

def map_ints_to_dmy(ints: tuple[int, int, int]) -> _DMY | None: ...
def map_ints_to_dm(ints: tuple[int, int]) -> _DM | None: ...
def two_to_four_digit_year(year: int) -> int: ...



#!/usr/bin/env python3
"""
Generates the protobuf stubs for the given s2clientprotocol version using mypy-protobuf.
Generally, new minor versions are a good time to update the stubs.
"""

from __future__ import annotations

import re
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path

from _utils import MYPY_PROTOBUF_VERSION, download_file, extract_archive, run_protoc
from ts_utils.metadata import update_metadata
from ts_utils.paths import distribution_path

# Whenever you update PACKAGE_VERSION here, version should be updated
# in stubs/s2clientprotocol/METADATA.toml and vice-versa.
PACKAGE_VERSION = "c04df4adbe274858a4eb8417175ee32ad02fd609"

STUBS_FOLDER = distribution_path("s2clientprotocol").absolute()
ARCHIVE_FILENAME = f"{PACKAGE_VERSION}.zip"
ARCHIVE_URL = f"https://github.com/Blizzard/s2client-proto/archive/{ARCHIVE_FILENAME}"
EXTRACTED_PACKAGE_DIR = f"s2client-proto-{PACKAGE_VERSION}"

VERSION_PATTERN = re.compile(r'def game_version\(\):\n    return "(.+?)"')


def extract_python_version(file_path: Path) -> str:
    """Extract Python version from s2clientprotocol's build file."""
    match = re.search(VERSION_PATTERN, file_path.read_text())
    assert match
    return match.group(1)


def main() -> None:
    temp_dir = Path(tempfile.mkdtemp())
    # Fetch s2clientprotocol (which contains all the .proto files)
    archive_path = temp_dir / ARCHIVE_FILENAME
    download_file(ARCHIVE_URL, archive_path)
    extract_archive(archive_path, temp_dir)

    # Remove existing pyi
    for old_stub in STUBS_FOLDER.rglob("*_pb2.pyi"):
        old_stub.unlink()

    protoc_version = run_protoc(
        proto_paths=(EXTRACTED_PACKAGE_DIR,),
        mypy_out=STUBS_FOLDER,
        proto_globs=(f"{EXTRACTED_PACKAGE_DIR}/s2clientprotocol/*.proto",),
        cwd=temp_dir,
    )

    python_s2_client_proto_version = extract_python_version(temp_dir / EXTRACTED_PACKAGE_DIR / "s2clientprotocol" / "build.py")

    # Cleanup after ourselves, this is a temp dir, but it can still grow fast if run multiple times
    shutil.rmtree(temp_dir)

    update_metadata(
        "s2clientprotocol",
        extra_description=f"""Partially generated using \
[mypy-protobuf=={MYPY_PROTOBUF_VERSION}](https://github.com/nipunn1313/mypy-protobuf/tree/v{MYPY_PROTOBUF_VERSION}) \
and {protoc_version} on \
[s2client-proto {python_s2_client_proto_version}](https://github.com/Blizzard/s2client-proto/tree/{PACKAGE_VERSION}).""",
    )
    print("Updated s2clientprotocol/METADATA.toml")

    # Run pre-commit to cleanup the stubs
    subprocess.run((sys.executable, "-m", "pre_commit", "run", "--files", *STUBS_FOLDER.rglob("*_pb2.pyi")), check=False)


if __name__ == "__main__":
    main()



from typing import Final

from braintree.resource import Resource

class EuropeBankAccount(Resource):
    class MandateType:
        Business: Final = "business"
        Consumer: Final = "consumer"

    @staticmethod
    def signature() -> list[str]: ...



from typing import Any

from .services import DriverConfig

class SwarmSpec(dict[str, Any]):
    def __init__(
        self,
        version: str,
        task_history_retention_limit: int | None = None,
        snapshot_interval: int | None = None,
        keep_old_snapshots: int | None = None,
        log_entries_for_slow_followers: int | None = None,
        heartbeat_tick: int | None = None,
        election_tick: int | None = None,
        dispatcher_heartbeat_period: int | None = None,
        node_cert_expiry: int | None = None,
        external_cas: list[SwarmExternalCA] | None = None,
        name: str | None = None,
        labels: dict[str, str] | None = None,
        signing_ca_cert: str | None = None,
        signing_ca_key: str | None = None,
        ca_force_rotate: int | None = None,
        autolock_managers: bool | None = None,
        log_driver: DriverConfig | None = None,
    ) -> None: ...

class SwarmExternalCA(dict[str, Any]):
    def __init__(
        self, url: str, protocol: str | None = None, options: dict[str, str] | None = None, ca_cert: str | None = None
    ) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping, Sequence

import requests
from docker.tls import TLSConfig
from requests.adapters import BaseAdapter

from .build import BuildApiMixin
from .config import ConfigApiMixin
from .container import ContainerApiMixin
from .daemon import DaemonApiMixin
from .exec_api import ExecApiMixin
from .image import ImageApiMixin
from .network import NetworkApiMixin
from .plugin import PluginApiMixin
from .secret import SecretApiMixin
from .service import ServiceApiMixin
from .swarm import SwarmApiMixin
from .volume import VolumeApiMixin

class APIClient(
    requests.Session,
    BuildApiMixin,
    ConfigApiMixin,
    ContainerApiMixin,
    DaemonApiMixin,
    ExecApiMixin,
    ImageApiMixin,
    NetworkApiMixin,
    PluginApiMixin,
    SecretApiMixin,
    ServiceApiMixin,
    SwarmApiMixin,
    VolumeApiMixin,
):
    __attrs__: Sequence[str]
    base_url: str
    timeout: int
    credstore_env: Mapping[Incomplete, Incomplete] | None
    def __init__(
        self,
        base_url: str | None = None,
        version: str | None = None,
        timeout: int = 60,
        tls: bool | TLSConfig = False,
        user_agent: str = ...,
        num_pools: int | None = None,
        credstore_env: Mapping[Incomplete, Incomplete] | None = None,
        use_ssh_client: bool = False,
        max_pool_size: int = 10,
    ) -> None: ...
    def get_adapter(self, url: str) -> BaseAdapter: ...
    @property
    def api_version(self) -> str: ...
    def reload_config(self, dockercfg_path: str | None = None) -> None: ...



import datetime
from typing import ClassVar

class VictoriaDayMixin:
    include_victoria_day: ClassVar[bool]
    victoria_day_label: ClassVar[str]

class VictoriaDayFourthMondayMay(VictoriaDayMixin):
    def get_victoria_day(self, year: int) -> tuple[datetime.date, str]: ...

class VictoriaDayLastMondayMay(VictoriaDayMixin):
    def get_victoria_day(self, year: int) -> tuple[datetime.date, str]: ...

class VictoriaDayFirstMondayJune(VictoriaDayMixin):
    def get_victoria_day(self, year: int) -> tuple[datetime.date, str]: ...



from logging import Logger

from paramiko.channel import Channel

CMD_INIT: int
CMD_VERSION: int
CMD_OPEN: int
CMD_CLOSE: int
CMD_READ: int
CMD_WRITE: int
CMD_LSTAT: int
CMD_FSTAT: int
CMD_SETSTAT: int
CMD_FSETSTAT: int
CMD_OPENDIR: int
CMD_READDIR: int
CMD_REMOVE: int
CMD_MKDIR: int
CMD_RMDIR: int
CMD_REALPATH: int
CMD_STAT: int
CMD_RENAME: int
CMD_READLINK: int
CMD_SYMLINK: int
CMD_STATUS: int
CMD_HANDLE: int
CMD_DATA: int
CMD_NAME: int
CMD_ATTRS: int
CMD_EXTENDED: int
CMD_EXTENDED_REPLY: int

SFTP_OK: int
SFTP_EOF: int
SFTP_NO_SUCH_FILE: int
SFTP_PERMISSION_DENIED: int
SFTP_FAILURE: int
SFTP_BAD_MESSAGE: int
SFTP_NO_CONNECTION: int
SFTP_CONNECTION_LOST: int
SFTP_OP_UNSUPPORTED: int

SFTP_DESC: list[str]

SFTP_FLAG_READ: int
SFTP_FLAG_WRITE: int
SFTP_FLAG_APPEND: int
SFTP_FLAG_CREATE: int
SFTP_FLAG_TRUNC: int
SFTP_FLAG_EXCL: int

CMD_NAMES: dict[int, str]

class int64(int): ...
class SFTPError(Exception): ...

class BaseSFTP:
    logger: Logger
    sock: Channel | None
    ultra_debug: bool
    def __init__(self) -> None: ...



from _typeshed import Incomplete, SupportsGetItem

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["closeness_centrality", "incremental_closeness_centrality"]

@_dispatchable
def closeness_centrality(
    G: Graph[_Node], u: _Node | None = None, distance=None, wf_improved: bool | None = True
) -> dict[_Node, float]: ...
@_dispatchable
def incremental_closeness_centrality(
    G: Graph[_Node],
    edge: tuple[Incomplete],
    prev_cc: SupportsGetItem[Incomplete, Incomplete] | None = None,
    insertion: bool | None = True,
    wf_improved: bool | None = True,
) -> dict[_Node, float]: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["from_graph6_bytes", "read_graph6", "to_graph6_bytes", "write_graph6"]

@_dispatchable
def from_graph6_bytes(bytes_in) -> Graph[Incomplete]: ...
def to_graph6_bytes(G: Graph[_Node], nodes=None, header: bool = True): ...
@_dispatchable
def read_graph6(path): ...
def write_graph6(G: Graph[_Node], path, nodes=None, header: bool = True): ...
def write_graph6_file(G: Graph[_Node], f, nodes: Iterable[Incomplete] | None = None, header: bool = True): ...
def data_to_n(data): ...
def n_to_data(n): ...



import pathlib
import re
from collections.abc import Generator
from datetime import datetime, timedelta, tzinfo
from typing import Final

class StaticTzInfo(tzinfo):
    def __init__(self, name: str, offset: timedelta) -> None: ...
    def tzname(self, dt) -> str: ...
    def utcoffset(self, dt) -> timedelta: ...
    def dst(self, dt) -> timedelta: ...
    def localize(self, dt: datetime, is_dst: bool = False) -> datetime: ...
    def __getinitargs__(self) -> tuple[str, timedelta]: ...

def pop_tz_offset_from_string(date_string: str, as_offset: bool = True) -> tuple[str, StaticTzInfo | str | None]: ...
def word_is_tz(word: str) -> bool: ...
def convert_to_local_tz(datetime_obj: datetime, datetime_tz_offset: timedelta) -> datetime: ...
def build_tz_offsets(search_regex_parts: list[str]) -> Generator[tuple[str, dict[str, re.Pattern[str] | timedelta]]]: ...
def get_local_tz_offset() -> timedelta: ...

local_tz_offset: timedelta

CACHE_PATH: Final[pathlib.Path]
current_hash: int | None



from typing import Final, Literal
from typing_extensions import TypeAlias

_Mode: TypeAlias = Literal[
    "GAE-LOCAL", "GAE-SERVER", "MOD_WSGI", "INTERACTIVE", "FCGI-FLUP", "UWSGI", "FLASK", "CHERRYPY", "BOTTLE", "UNKNOWN"
]
__version__: Final[str]

def detect_mode() -> _Mode: ...

MODE: Final[_Mode]
MAIN_FILE: Final[str | None]
MAIN_DIR: Final[str | None]
PONY_DIR: Final[str]



from _typeshed import Incomplete, StrPath
from typing import Final
from typing_extensions import deprecated

from docutils import SettingsSpec
from docutils.io import FileInput, Input, Output
from docutils.parsers import Parser
from docutils.readers import Reader
from docutils.utils import SystemMessage
from docutils.writers import Writer, _WriterParts

__docformat__: Final = "reStructuredText"

class Publisher:
    document: Incomplete | None
    reader: Reader[Incomplete]
    parser: Parser
    writer: Writer[Incomplete]
    source: Input[Incomplete]
    source_class: Incomplete
    destination: Output | None
    destination_class: Incomplete
    settings: dict[str, Incomplete]
    def __init__(
        self,
        reader: Reader[Incomplete] | None = None,
        parser: Parser | None = None,
        writer: Writer[Incomplete] | None = None,
        source: Input[Incomplete] | None = None,
        source_class=...,
        destination: Output | None = None,
        destination_class=...,
        settings: dict[str, Incomplete] | None = None,
    ) -> None: ...
    def set_reader(self, reader: str, parser: Parser | None = None, parser_name: str | None = None) -> None: ...
    def set_writer(self, writer_name: str) -> None: ...
    @deprecated("The `Publisher.set_components()` will be removed in Docutils 2.0.")
    def set_components(self, reader_name: str, parser_name: str, writer_name: str) -> None: ...
    def get_settings(
        self,
        usage: str | None = None,
        description: str | None = None,
        settings_spec: SettingsSpec | None = None,
        config_section: str | None = None,
        **defaults,
    ): ...
    def process_programmatic_settings(self, settings_spec, settings_overrides, config_section) -> None: ...
    def process_command_line(
        self,
        argv: list[str] | None = None,
        usage=None,
        description: str | None = None,
        settings_spec=None,
        config_section=None,
        **defaults,
    ) -> None: ...
    def set_io(self, source_path: StrPath | None = None, destination_path: StrPath | None = None) -> None: ...
    def set_source(self, source: str | None = None, source_path: StrPath | None = None) -> None: ...
    def set_destination(self, destination=None, destination_path: StrPath | None = None) -> None: ...
    def apply_transforms(self) -> None: ...
    def publish(
        self,
        argv: list[str] | None = None,
        usage: str | None = None,
        description: str | None = None,
        settings_spec=None,
        settings_overrides=None,
        config_section: str | None = None,
        enable_exit_status: bool = False,
    ): ...
    def debugging_dumps(self) -> None: ...
    def prompt(self) -> None: ...
    def report_Exception(self, error: BaseException) -> None: ...
    def report_SystemMessage(self, error: SystemMessage) -> None: ...
    def report_UnicodeError(self, error: UnicodeEncodeError) -> None: ...

default_usage: Final[str]
default_description: Final[str]

def publish_cmdline(
    reader: Reader[Incomplete] | None = None,
    reader_name: str | None = None,
    parser: Parser | None = None,
    parser_name: str | None = None,
    writer: Writer[Incomplete] | None = None,
    writer_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = True,
    argv: list[str] | None = None,
    usage: str = "%prog [options] [<source> [<destination>]]",
    description: str = ...,
): ...
def publish_file(
    source=None,
    source_path: StrPath | None = None,
    destination=None,
    destination_path: StrPath | None = None,
    reader=None,
    reader_name: str | None = None,
    parser=None,
    parser_name: str | None = None,
    writer=None,
    writer_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = False,
): ...
def publish_string(
    source,
    source_path: StrPath | None = None,
    destination_path: StrPath | None = None,
    reader=None,
    reader_name: str | None = None,
    parser=None,
    parser_name: str | None = None,
    writer=None,
    writer_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = False,
): ...
def publish_parts(
    source,
    source_path: StrPath | None = None,
    source_class=...,
    destination_path: StrPath | None = None,
    reader=None,
    reader_name: str | None = None,
    parser=None,
    parser_name: str | None = None,
    writer=None,
    writer_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides: dict[str, Incomplete] | None = None,
    config_section: str | None = None,
    enable_exit_status: bool = False,
) -> _WriterParts: ...
def publish_doctree(
    source,
    source_path: StrPath | None = None,
    source_class=...,
    reader=None,
    reader_name: str | None = None,
    parser=None,
    parser_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = False,
): ...
def publish_from_doctree(
    document,
    destination_path: StrPath | None = None,
    writer=None,
    writer_name: str | None = None,
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = False,
): ...
@deprecated("The `publish_cmdline_to_binary()` is deprecated  by `publish_cmdline()` and will be removed in Docutils 0.24.")
def publish_cmdline_to_binary(
    reader=None,
    reader_name: str = "standalone",
    parser=None,
    parser_name: str = "restructuredtext",
    writer=None,
    writer_name: str = "pseudoxml",
    settings=None,
    settings_spec=None,
    settings_overrides=None,
    config_section: str | None = None,
    enable_exit_status: bool = True,
    argv: list[str] | None = None,
    usage: str = "%prog [options] [<source> [<destination>]]",
    description: str = ...,
    destination=None,
    destination_class=...,
): ...
def publish_programmatically(
    source_class: type[FileInput],
    source,
    source_path: StrPath | None,
    destination_class,
    destination,
    destination_path: StrPath | None,
    reader,
    reader_name: str,
    parser,
    parser_name: str,
    writer,
    writer_name: str,
    settings,
    settings_spec,
    settings_overrides,
    config_section: str,
    enable_exit_status: bool,
) -> tuple[str | bytes | None, Publisher]: ...
def rst2something(writer: str, documenttype: str, doc_path: str = "") -> None: ...
def rst2html() -> None: ...
def rst2html4() -> None: ...
def rst2html5() -> None: ...
def rst2latex() -> None: ...
def rst2man() -> None: ...
def rst2odt() -> None: ...
def rst2pseudoxml() -> None: ...
def rst2s5() -> None: ...
def rst2xetex() -> None: ...
def rst2xml() -> None: ...



from collections.abc import Callable
from typing import Any, ClassVar
from typing_extensions import TypeAlias

from channels.routing import ProtocolTypeRouter
from channels.utils import _ChannelApplication
from django.contrib.staticfiles.handlers import ASGIStaticFilesHandler
from django.test.testcases import TransactionTestCase

DaphneProcess: TypeAlias = Any  # TODO: temporary hack for daphne.testing.DaphneProcess; remove once daphne provides types

_StaticWrapper: TypeAlias = Callable[[ProtocolTypeRouter], _ChannelApplication]

def make_application(*, static_wrapper: _StaticWrapper | None) -> Any: ...
def set_database_connection() -> None: ...

class ChannelsLiveServerTestCase(TransactionTestCase):
    host: ClassVar[str] = "localhost"
    ProtocolServerProcess: ClassVar[type[DaphneProcess]] = ...
    static_wrapper: ClassVar[type[ASGIStaticFilesHandler]] = ...
    serve_static: ClassVar[bool] = True

    @property
    def live_server_url(self) -> str: ...
    @property
    def live_server_ws_url(self) -> str: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.us_bank_account_verification import UsBankAccountVerification

class UsBankAccountVerificationGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def confirm_micro_transfer_amounts(self, verification_id: str, amounts) -> SuccessfulResult | ErrorResult | None: ...
    def find(self, verification_id: str) -> UsBankAccountVerification: ...
    def search(self, *query) -> ResourceCollection: ...



from collections.abc import Callable

from oauthlib.openid.connect.core.request_validator import RequestValidator

from .device_authorization import DeviceAuthorizationEndpoint

class DeviceApplicationServer(DeviceAuthorizationEndpoint):
    def __init__(
        self,
        request_validator: RequestValidator,
        verification_uri: str,
        interval: int = 5,
        verification_uri_complete: str | None = None,
        user_code_generator: Callable[[None], str] | None = None,
        **kwargs,
    ) -> None: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult

class PayPalPaymentResource(Resource):
    def __init__(self, gateway, attributes) -> None: ...
    @staticmethod
    def update(request) -> SuccessfulResult | ErrorResult: ...
    @staticmethod
    def update_signature() -> list[Incomplete]: ...



from _typeshed import Incomplete

class ExceptionFSM(Exception):
    value: Incomplete
    def __init__(self, value) -> None: ...

class FSM:
    state_transitions: Incomplete
    state_transitions_any: Incomplete
    default_transition: Incomplete
    input_symbol: Incomplete
    initial_state: Incomplete
    current_state: Incomplete
    next_state: Incomplete
    action: Incomplete
    memory: Incomplete
    def __init__(self, initial_state, memory=None) -> None: ...
    def reset(self) -> None: ...
    def add_transition(self, input_symbol, state, action=None, next_state=None) -> None: ...
    def add_transition_list(self, list_input_symbols, state, action=None, next_state=None) -> None: ...
    def add_transition_any(self, state, action=None, next_state=None) -> None: ...
    def set_default_transition(self, action, next_state) -> None: ...
    def get_transition(self, input_symbol, state): ...
    def process(self, input_symbol) -> None: ...
    def process_list(self, input_symbols) -> None: ...

PY3: Incomplete

def BeginBuildNumber(fsm) -> None: ...
def BuildNumber(fsm) -> None: ...
def EndBuildNumber(fsm) -> None: ...
def DoOperator(fsm) -> None: ...
def DoEqual(fsm) -> None: ...
def Error(fsm) -> None: ...
def main() -> None: ...



from typing import Final, Literal

EPOCH: Final = 2374479.5
YEAR_EPOCH: Final = 1789
DAYS_IN_YEAR: Final = 365
MONTHS: Final[tuple[str, ...]]

def legal_date(year: int, month: int, day: int) -> Literal[True]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def dayname(year: int, month: int, day: int) -> tuple[str, str]: ...
def weekday(day: int) -> int: ...
def festival(month: int, day: int) -> str | None: ...



from docker import APIClient

class TLSConfig:
    cert: tuple[str, str] | None
    ca_cert: str | None
    verify: bool | str | None
    def __init__(
        self, client_cert: tuple[str, str] | None = None, ca_cert: str | None = None, verify: bool | str | None = None
    ) -> None: ...
    def configure_client(self, client: APIClient) -> None: ...



import calendar
import datetime
from collections.abc import Generator
from typing import Literal, TypedDict, overload, type_check_only
from typing_extensions import Self

from .dates import BaseDate, HebrewDate

@type_check_only
class _MoladDict(TypedDict):
    weekday: int
    hours: int
    parts: int

@type_check_only
class _MoladAnnouncementDict(TypedDict):
    weekday: int
    hour: int
    minutes: int
    parts: int

class IllegalMonthError(ValueError):
    month: int
    def __init__(self, month: int) -> None: ...

class IllegalWeekdayError(ValueError):
    weekday: int
    def __init__(self, weekday: int) -> None: ...

class Year:
    year: int
    leap: bool
    def __init__(self, year: int) -> None: ...
    def __len__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __add__(self, other: int) -> Year: ...
    @overload
    def __sub__(self, other: int) -> Year: ...
    @overload
    def __sub__(self, other: Year) -> int: ...
    def __gt__(self, other: Year) -> bool: ...
    def __ge__(self, other: Year) -> bool: ...
    def __lt__(self, other: Year) -> bool: ...
    def __le__(self, other: Year) -> bool: ...
    def __iter__(self) -> Generator[int]: ...
    def monthscount(self) -> Literal[12, 13]: ...
    def itermonths(self) -> Generator[Month]: ...
    def iterdays(self) -> Generator[int]: ...
    def iterdates(self) -> Generator[HebrewDate]: ...
    @classmethod
    def from_date(cls, date: BaseDate) -> Self: ...
    @classmethod
    def from_pydate(cls, pydate: datetime.date) -> Self: ...
    def year_string(self, thousands: bool = False) -> str: ...

class Month:
    year: int
    month: int
    def __init__(self, year: int, month: int) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Generator[int]: ...
    def __eq__(self, other: object) -> bool: ...
    def __add__(self, other: int) -> Month: ...
    @overload
    def __sub__(self, other: int) -> Month: ...
    @overload
    def __sub__(self, other: Month) -> int: ...
    def __gt__(self, other: Month) -> bool: ...
    def __ge__(self, other: Month) -> bool: ...
    def __lt__(self, other: Month) -> bool: ...
    def __le__(self, other: Month) -> bool: ...
    @classmethod
    def from_date(cls, date: BaseDate) -> Month: ...
    @classmethod
    def from_pydate(cls, pydate: datetime.date) -> Month: ...
    def month_name(self, hebrew: bool = False) -> str: ...
    def month_string(self, thousands: bool = False) -> str: ...
    def starting_weekday(self) -> int: ...
    def iterdates(self) -> Generator[HebrewDate]: ...
    def molad(self) -> _MoladDict: ...
    def molad_announcement(self) -> _MoladAnnouncementDict: ...

def to_hebrew_numeral(num: int, thousands: bool = False, withgershayim: bool = True) -> str: ...

class HebrewCalendar(calendar.Calendar):
    hebrewnumerals: bool
    hebrewweekdays: bool
    hebrewmonths: bool
    hebrewyear: bool
    def __init__(
        self,
        firstweekday: int = 1,
        hebrewnumerals: bool = True,
        hebrewweekdays: bool = False,
        hebrewmonths: bool = False,
        hebrewyear: bool = False,
    ) -> None: ...
    @property
    def firstweekday(self) -> int: ...
    @firstweekday.setter
    def firstweekday(self, thefirstweekday: int) -> None: ...
    def iterweekdays(self) -> Generator[int]: ...
    def itermonthdates(self, year: int, month: int) -> Generator[HebrewDate]: ...  # type: ignore[override]
    def itermonthdays(self, year: int, month: int) -> Generator[int]: ...
    def itermonthdays2(self, year: int, month: int) -> Generator[tuple[int, int]]: ...
    def itermonthdays3(self, year: int, month: int) -> Generator[tuple[int, int, int]]: ...
    def itermonthdays4(self, year: int, month: int) -> Generator[tuple[int, int, int, int]]: ...
    def yeardatescalendar(self, year: int, width: int = 3) -> list[list[list[list[HebrewDate]]]]: ...  # type: ignore[override]
    def yeardays2calendar(self, year: int, width: int = 3) -> list[list[list[list[tuple[int, int]]]]]: ...
    def yeardayscalendar(self, year: int, width: int = 3) -> list[list[list[list[int]]]]: ...
    def monthdatescalendar(self, year: int, month: int) -> list[list[HebrewDate]]: ...  # type: ignore[override]

class HebrewHTMLCalendar(HebrewCalendar, calendar.HTMLCalendar):
    rtl: bool
    def __init__(
        self,
        firstweekday: int = 1,
        hebrewnumerals: bool = True,
        hebrewweekdays: bool = False,
        hebrewmonths: bool = False,
        hebrewyear: bool = False,
        rtl: bool = False,
    ) -> None: ...
    def formatday(self, day: int, weekday: int) -> str: ...
    def formatweekday(self, day: int) -> str: ...
    def formatyearnumber(self, theyear: int) -> int | str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    def formatmonth(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    def formatyear(self, theyear: int, width: int = 3) -> str: ...

class HebrewTextCalendar(HebrewCalendar, calendar.TextCalendar):
    def formatday(self, day: int, weekday: int, width: int) -> str: ...
    def formatweekday(self, day: int, width: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, width: int = 0, withyear: bool = True) -> str: ...
    def formatyear(self, theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...

def fast_day(date: BaseDate, hebrew: bool = False) -> str | None: ...
def festival(
    date: BaseDate, israel: bool = False, hebrew: bool = False, include_working_days: bool = True, prefix_day: bool = False
) -> str | None: ...
def holiday(date: BaseDate, israel: bool = False, hebrew: bool = False, prefix_day: bool = False) -> str | None: ...



from typing import Final

def lxml_available() -> bool: ...
def lxml_env_set() -> bool: ...

LXML: Final[bool]

def defusedxml_available() -> bool: ...
def defusedxml_env_set() -> bool: ...

DEFUSEDXML: Final[bool]



from types import ModuleType

__all__ = ["getTreeWalker", "pprint"]

def getTreeWalker(treeType: str, implementation: ModuleType | None = None, **kwargs): ...
def concatenateCharacterTokens(tokens): ...
def pprint(walker) -> str: ...



from _typeshed import Incomplete
from builtins import list as _list

from ..rest import RestClientOptions
from ..types import TimeoutType

class Roles:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def list(
        self, page: int = 0, per_page: int = 25, include_totals: bool = True, name_filter: str | None = None
    ) -> dict[str, Incomplete]: ...
    async def list_async(
        self, page: int = 0, per_page: int = 25, include_totals: bool = True, name_filter: str | None = None
    ) -> dict[str, Incomplete]: ...
    def create(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get(self, id: str) -> dict[str, Incomplete]: ...
    async def get_async(self, id: str) -> dict[str, Incomplete]: ...
    def delete(self, id: str): ...
    async def delete_async(self, id: str): ...
    def update(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def list_users(
        self,
        id: str,
        page: int = 0,
        per_page: int = 25,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
    ) -> dict[str, Incomplete]: ...
    async def list_users_async(
        self,
        id: str,
        page: int = 0,
        per_page: int = 25,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
    ) -> dict[str, Incomplete]: ...
    def add_users(self, id: str, users: _list[str]) -> dict[str, Incomplete]: ...
    async def add_users_async(self, id: str, users: _list[str]) -> dict[str, Incomplete]: ...
    def list_permissions(
        self, id: str, page: int = 0, per_page: int = 25, include_totals: bool = True
    ) -> dict[str, Incomplete]: ...
    async def list_permissions_async(
        self, id: str, page: int = 0, per_page: int = 25, include_totals: bool = True
    ) -> dict[str, Incomplete]: ...
    def remove_permissions(self, id: str, permissions: _list[dict[str, str]]): ...
    async def remove_permissions_async(self, id: str, permissions: _list[dict[str, str]]): ...
    def add_permissions(self, id: str, permissions: _list[dict[str, str]]) -> dict[str, Incomplete]: ...
    async def add_permissions_async(self, id: str, permissions: _list[dict[str, str]]) -> dict[str, Incomplete]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import s2clientprotocol.common_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Attribute:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AttributeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Attribute.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Light: _Attribute.ValueType  # 1
    Armored: _Attribute.ValueType  # 2
    Biological: _Attribute.ValueType  # 3
    Mechanical: _Attribute.ValueType  # 4
    Robotic: _Attribute.ValueType  # 5
    Psionic: _Attribute.ValueType  # 6
    Massive: _Attribute.ValueType  # 7
    Structure: _Attribute.ValueType  # 8
    Hover: _Attribute.ValueType  # 9
    Heroic: _Attribute.ValueType  # 10
    Summoned: _Attribute.ValueType  # 11

class Attribute(_Attribute, metaclass=_AttributeEnumTypeWrapper): ...

Light: Attribute.ValueType  # 1
Armored: Attribute.ValueType  # 2
Biological: Attribute.ValueType  # 3
Mechanical: Attribute.ValueType  # 4
Robotic: Attribute.ValueType  # 5
Psionic: Attribute.ValueType  # 6
Massive: Attribute.ValueType  # 7
Structure: Attribute.ValueType  # 8
Hover: Attribute.ValueType  # 9
Heroic: Attribute.ValueType  # 10
Summoned: Attribute.ValueType  # 11
global___Attribute = Attribute

@typing.final
class AbilityData(google.protobuf.message.Message):
    """May not relevant: queueable (everything is queueable).
    May not be important: AbilSetId - marine stim, marauder stim.
    Stuff omitted: transient.
    Stuff that may be important: cost, range, Alignment, targetfilters.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Target:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TargetEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[AbilityData._Target.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Point: AbilityData._Target.ValueType  # 2
        """Requires a target position."""
        Unit: AbilityData._Target.ValueType  # 3
        """Requires a unit to target. Given by position using feature layers."""
        PointOrUnit: AbilityData._Target.ValueType  # 4
        """Requires either a target point or target unit."""
        PointOrNone: AbilityData._Target.ValueType  # 5
        """Requires either a target point or no target. (eg. building add-ons)"""

    class Target(_Target, metaclass=_TargetEnumTypeWrapper): ...
    Point: AbilityData.Target.ValueType  # 2
    """Requires a target position."""
    Unit: AbilityData.Target.ValueType  # 3
    """Requires a unit to target. Given by position using feature layers."""
    PointOrUnit: AbilityData.Target.ValueType  # 4
    """Requires either a target point or target unit."""
    PointOrNone: AbilityData.Target.ValueType  # 5
    """Requires either a target point or no target. (eg. building add-ons)"""

    ABILITY_ID_FIELD_NUMBER: builtins.int
    LINK_NAME_FIELD_NUMBER: builtins.int
    LINK_INDEX_FIELD_NUMBER: builtins.int
    BUTTON_NAME_FIELD_NUMBER: builtins.int
    FRIENDLY_NAME_FIELD_NUMBER: builtins.int
    HOTKEY_FIELD_NUMBER: builtins.int
    REMAPS_TO_ABILITY_ID_FIELD_NUMBER: builtins.int
    AVAILABLE_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    ALLOW_MINIMAP_FIELD_NUMBER: builtins.int
    ALLOW_AUTOCAST_FIELD_NUMBER: builtins.int
    IS_BUILDING_FIELD_NUMBER: builtins.int
    FOOTPRINT_RADIUS_FIELD_NUMBER: builtins.int
    IS_INSTANT_PLACEMENT_FIELD_NUMBER: builtins.int
    CAST_RANGE_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    """Stable ID."""
    link_name: builtins.str
    """Catalog name of the ability."""
    link_index: builtins.int
    """Catalog index of the ability."""
    button_name: builtins.str
    """Name used for the command card. May not always be set."""
    friendly_name: builtins.str
    """A human friendly name when the button name or link name isn't descriptive."""
    hotkey: builtins.str
    """Hotkey. May not always be set."""
    remaps_to_ability_id: builtins.int
    """This ability id may be represented by the given more generic id."""
    available: builtins.bool
    """If true, the ability may be used by this set of mods/map."""
    target: global___AbilityData.Target.ValueType
    """Determines if a point is optional or required."""
    allow_minimap: builtins.bool
    """Can be cast in the minimap."""
    allow_autocast: builtins.bool
    """Autocast can be set."""
    is_building: builtins.bool
    """Requires placement to construct a building."""
    footprint_radius: builtins.float
    """Estimation of the footprint size. Need a better footprint."""
    is_instant_placement: builtins.bool
    """Placement next to an existing structure, e.g., an add-on like a Tech Lab."""
    cast_range: builtins.float
    """Range unit can cast ability without needing to approach target."""
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        link_name: builtins.str | None = ...,
        link_index: builtins.int | None = ...,
        button_name: builtins.str | None = ...,
        friendly_name: builtins.str | None = ...,
        hotkey: builtins.str | None = ...,
        remaps_to_ability_id: builtins.int | None = ...,
        available: builtins.bool | None = ...,
        target: global___AbilityData.Target.ValueType | None = ...,
        allow_minimap: builtins.bool | None = ...,
        allow_autocast: builtins.bool | None = ...,
        is_building: builtins.bool | None = ...,
        footprint_radius: builtins.float | None = ...,
        is_instant_placement: builtins.bool | None = ...,
        cast_range: builtins.float | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "allow_autocast",
            b"allow_autocast",
            "allow_minimap",
            b"allow_minimap",
            "available",
            b"available",
            "button_name",
            b"button_name",
            "cast_range",
            b"cast_range",
            "footprint_radius",
            b"footprint_radius",
            "friendly_name",
            b"friendly_name",
            "hotkey",
            b"hotkey",
            "is_building",
            b"is_building",
            "is_instant_placement",
            b"is_instant_placement",
            "link_index",
            b"link_index",
            "link_name",
            b"link_name",
            "remaps_to_ability_id",
            b"remaps_to_ability_id",
            "target",
            b"target",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "allow_autocast",
            b"allow_autocast",
            "allow_minimap",
            b"allow_minimap",
            "available",
            b"available",
            "button_name",
            b"button_name",
            "cast_range",
            b"cast_range",
            "footprint_radius",
            b"footprint_radius",
            "friendly_name",
            b"friendly_name",
            "hotkey",
            b"hotkey",
            "is_building",
            b"is_building",
            "is_instant_placement",
            b"is_instant_placement",
            "link_index",
            b"link_index",
            "link_name",
            b"link_name",
            "remaps_to_ability_id",
            b"remaps_to_ability_id",
            "target",
            b"target",
        ],
    ) -> None: ...

global___AbilityData = AbilityData

@typing.final
class DamageBonus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ATTRIBUTE_FIELD_NUMBER: builtins.int
    BONUS_FIELD_NUMBER: builtins.int
    attribute: global___Attribute.ValueType
    bonus: builtins.float
    def __init__(self, *, attribute: global___Attribute.ValueType | None = ..., bonus: builtins.float | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["attribute", b"attribute", "bonus", b"bonus"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attribute", b"attribute", "bonus", b"bonus"]) -> None: ...

global___DamageBonus = DamageBonus

@typing.final
class Weapon(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _TargetType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TargetTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Weapon._TargetType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Ground: Weapon._TargetType.ValueType  # 1
        Air: Weapon._TargetType.ValueType  # 2
        Any: Weapon._TargetType.ValueType  # 3

    class TargetType(_TargetType, metaclass=_TargetTypeEnumTypeWrapper): ...
    Ground: Weapon.TargetType.ValueType  # 1
    Air: Weapon.TargetType.ValueType  # 2
    Any: Weapon.TargetType.ValueType  # 3

    TYPE_FIELD_NUMBER: builtins.int
    DAMAGE_FIELD_NUMBER: builtins.int
    DAMAGE_BONUS_FIELD_NUMBER: builtins.int
    ATTACKS_FIELD_NUMBER: builtins.int
    RANGE_FIELD_NUMBER: builtins.int
    SPEED_FIELD_NUMBER: builtins.int
    type: global___Weapon.TargetType.ValueType
    damage: builtins.float
    attacks: builtins.int
    """Number of hits per attack. (eg. Colossus has 2 beams)"""
    range: builtins.float
    speed: builtins.float
    """Time between attacks."""
    @property
    def damage_bonus(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DamageBonus]: ...
    def __init__(
        self,
        *,
        type: global___Weapon.TargetType.ValueType | None = ...,
        damage: builtins.float | None = ...,
        damage_bonus: collections.abc.Iterable[global___DamageBonus] | None = ...,
        attacks: builtins.int | None = ...,
        range: builtins.float | None = ...,
        speed: builtins.float | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "attacks", b"attacks", "damage", b"damage", "range", b"range", "speed", b"speed", "type", b"type"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "attacks",
            b"attacks",
            "damage",
            b"damage",
            "damage_bonus",
            b"damage_bonus",
            "range",
            b"range",
            "speed",
            b"speed",
            "type",
            b"type",
        ],
    ) -> None: ...

global___Weapon = Weapon

@typing.final
class UnitTypeData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    AVAILABLE_FIELD_NUMBER: builtins.int
    CARGO_SIZE_FIELD_NUMBER: builtins.int
    MINERAL_COST_FIELD_NUMBER: builtins.int
    VESPENE_COST_FIELD_NUMBER: builtins.int
    FOOD_REQUIRED_FIELD_NUMBER: builtins.int
    FOOD_PROVIDED_FIELD_NUMBER: builtins.int
    ABILITY_ID_FIELD_NUMBER: builtins.int
    RACE_FIELD_NUMBER: builtins.int
    BUILD_TIME_FIELD_NUMBER: builtins.int
    HAS_VESPENE_FIELD_NUMBER: builtins.int
    HAS_MINERALS_FIELD_NUMBER: builtins.int
    SIGHT_RANGE_FIELD_NUMBER: builtins.int
    TECH_ALIAS_FIELD_NUMBER: builtins.int
    UNIT_ALIAS_FIELD_NUMBER: builtins.int
    TECH_REQUIREMENT_FIELD_NUMBER: builtins.int
    REQUIRE_ATTACHED_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    MOVEMENT_SPEED_FIELD_NUMBER: builtins.int
    ARMOR_FIELD_NUMBER: builtins.int
    WEAPONS_FIELD_NUMBER: builtins.int
    unit_id: builtins.int
    """Stable ID."""
    name: builtins.str
    """Catalog name of the unit."""
    available: builtins.bool
    """If true, the ability may be used by this set of mods/map."""
    cargo_size: builtins.int
    """Number of cargo slots it occupies in transports."""
    mineral_cost: builtins.int
    vespene_cost: builtins.int
    food_required: builtins.float
    food_provided: builtins.float
    ability_id: builtins.int
    """The ability that builds this unit."""
    race: s2clientprotocol.common_pb2.Race.ValueType
    build_time: builtins.float
    has_vespene: builtins.bool
    has_minerals: builtins.bool
    sight_range: builtins.float
    """Range unit reveals vision."""
    unit_alias: builtins.int
    """The morphed variant of this unit."""
    tech_requirement: builtins.int
    """Structure required to build this unit. (Or any with the same tech_alias)"""
    require_attached: builtins.bool
    """Whether tech_requirement is an add-on."""
    movement_speed: builtins.float
    armor: builtins.float
    @property
    def tech_alias(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Other units that satisfy the same tech requirement."""

    @property
    def attributes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___Attribute.ValueType]:
        """Values include changes from upgrades"""

    @property
    def weapons(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Weapon]: ...
    def __init__(
        self,
        *,
        unit_id: builtins.int | None = ...,
        name: builtins.str | None = ...,
        available: builtins.bool | None = ...,
        cargo_size: builtins.int | None = ...,
        mineral_cost: builtins.int | None = ...,
        vespene_cost: builtins.int | None = ...,
        food_required: builtins.float | None = ...,
        food_provided: builtins.float | None = ...,
        ability_id: builtins.int | None = ...,
        race: s2clientprotocol.common_pb2.Race.ValueType | None = ...,
        build_time: builtins.float | None = ...,
        has_vespene: builtins.bool | None = ...,
        has_minerals: builtins.bool | None = ...,
        sight_range: builtins.float | None = ...,
        tech_alias: collections.abc.Iterable[builtins.int] | None = ...,
        unit_alias: builtins.int | None = ...,
        tech_requirement: builtins.int | None = ...,
        require_attached: builtins.bool | None = ...,
        attributes: collections.abc.Iterable[global___Attribute.ValueType] | None = ...,
        movement_speed: builtins.float | None = ...,
        armor: builtins.float | None = ...,
        weapons: collections.abc.Iterable[global___Weapon] | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "armor",
            b"armor",
            "available",
            b"available",
            "build_time",
            b"build_time",
            "cargo_size",
            b"cargo_size",
            "food_provided",
            b"food_provided",
            "food_required",
            b"food_required",
            "has_minerals",
            b"has_minerals",
            "has_vespene",
            b"has_vespene",
            "mineral_cost",
            b"mineral_cost",
            "movement_speed",
            b"movement_speed",
            "name",
            b"name",
            "race",
            b"race",
            "require_attached",
            b"require_attached",
            "sight_range",
            b"sight_range",
            "tech_requirement",
            b"tech_requirement",
            "unit_alias",
            b"unit_alias",
            "unit_id",
            b"unit_id",
            "vespene_cost",
            b"vespene_cost",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "armor",
            b"armor",
            "attributes",
            b"attributes",
            "available",
            b"available",
            "build_time",
            b"build_time",
            "cargo_size",
            b"cargo_size",
            "food_provided",
            b"food_provided",
            "food_required",
            b"food_required",
            "has_minerals",
            b"has_minerals",
            "has_vespene",
            b"has_vespene",
            "mineral_cost",
            b"mineral_cost",
            "movement_speed",
            b"movement_speed",
            "name",
            b"name",
            "race",
            b"race",
            "require_attached",
            b"require_attached",
            "sight_range",
            b"sight_range",
            "tech_alias",
            b"tech_alias",
            "tech_requirement",
            b"tech_requirement",
            "unit_alias",
            b"unit_alias",
            "unit_id",
            b"unit_id",
            "vespene_cost",
            b"vespene_cost",
            "weapons",
            b"weapons",
        ],
    ) -> None: ...

global___UnitTypeData = UnitTypeData

@typing.final
class UpgradeData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UPGRADE_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    MINERAL_COST_FIELD_NUMBER: builtins.int
    VESPENE_COST_FIELD_NUMBER: builtins.int
    RESEARCH_TIME_FIELD_NUMBER: builtins.int
    ABILITY_ID_FIELD_NUMBER: builtins.int
    upgrade_id: builtins.int
    """Stable ID."""
    name: builtins.str
    mineral_cost: builtins.int
    vespene_cost: builtins.int
    research_time: builtins.float
    ability_id: builtins.int
    def __init__(
        self,
        *,
        upgrade_id: builtins.int | None = ...,
        name: builtins.str | None = ...,
        mineral_cost: builtins.int | None = ...,
        vespene_cost: builtins.int | None = ...,
        research_time: builtins.float | None = ...,
        ability_id: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "mineral_cost",
            b"mineral_cost",
            "name",
            b"name",
            "research_time",
            b"research_time",
            "upgrade_id",
            b"upgrade_id",
            "vespene_cost",
            b"vespene_cost",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "ability_id",
            b"ability_id",
            "mineral_cost",
            b"mineral_cost",
            "name",
            b"name",
            "research_time",
            b"research_time",
            "upgrade_id",
            b"upgrade_id",
            "vespene_cost",
            b"vespene_cost",
        ],
    ) -> None: ...

global___UpgradeData = UpgradeData

@typing.final
class BuffData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUFF_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    buff_id: builtins.int
    """Stable ID."""
    name: builtins.str
    def __init__(self, *, buff_id: builtins.int | None = ..., name: builtins.str | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["buff_id", b"buff_id", "name", b"name"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buff_id", b"buff_id", "name", b"name"]) -> None: ...

global___BuffData = BuffData

@typing.final
class EffectData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EFFECT_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    FRIENDLY_NAME_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    effect_id: builtins.int
    """Stable ID."""
    name: builtins.str
    friendly_name: builtins.str
    radius: builtins.float
    def __init__(
        self,
        *,
        effect_id: builtins.int | None = ...,
        name: builtins.str | None = ...,
        friendly_name: builtins.str | None = ...,
        radius: builtins.float | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "effect_id", b"effect_id", "friendly_name", b"friendly_name", "name", b"name", "radius", b"radius"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "effect_id", b"effect_id", "friendly_name", b"friendly_name", "name", b"name", "radius", b"radius"
        ],
    ) -> None: ...

global___EffectData = EffectData



"""Preconfigured converters for pyyaml."""

from cattrs.preconf.pyyaml import PyyamlConverter, configure_converter, make_converter

__all__ = ["configure_converter", "make_converter", "PyyamlConverter"]



"""Jaseci utility functions and libraries."""

import importlib

# lang_tools.py remains here as it's a high-level tool with many dependencies
# NonGPT.jac and symtable_test_helpers.jac remain here as well


def __getattr__(name: str) -> object:
    """Lazy load .jac modules."""
    if name == "NonGPT":
        # Import the NonGPT module (which is a .jac file)
        return importlib.import_module("jaclang.utils.NonGPT")
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")



from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.subscription import Subscription
from braintree.successful_result import SuccessfulResult

class PayPalAccount(Resource):
    @staticmethod
    def find(paypal_account_token: str) -> PayPalAccount | None: ...
    @staticmethod
    def delete(paypal_account_token: str) -> SuccessfulResult: ...
    @staticmethod
    def update(paypal_account_token: str, params=None) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def signature() -> list[str | dict[str, list[str]]]: ...
    subscriptions: list[Subscription]
    def __init__(self, gateway, attributes) -> None: ...



from __future__ import annotations

from typing import Optional, cast
from typing_extensions import assert_type

import grpc

# Channel options:
assert_type(grpc.insecure_channel("target", ()), grpc.Channel)
assert_type(grpc.insecure_channel("target", (("a", "b"),)), grpc.Channel)
assert_type(grpc.insecure_channel("target", (("a", "b"), ("c", "d"))), grpc.Channel)

# Local channel credentials:
creds = grpc.local_channel_credentials(grpc.LocalConnectionType.LOCAL_TCP)
assert_type(creds, grpc.ChannelCredentials)

# Other credential types:
assert_type(grpc.alts_channel_credentials(), grpc.ChannelCredentials)
assert_type(grpc.alts_server_credentials(), grpc.ServerCredentials)
assert_type(grpc.compute_engine_channel_credentials(grpc.CallCredentials("")), grpc.ChannelCredentials)
assert_type(grpc.insecure_server_credentials(), grpc.ServerCredentials)

# XDS credentials:
assert_type(
    grpc.xds_channel_credentials(grpc.local_channel_credentials(grpc.LocalConnectionType.LOCAL_TCP)), grpc.ChannelCredentials
)
assert_type(grpc.xds_server_credentials(grpc.insecure_server_credentials()), grpc.ServerCredentials)

# Channel ready future
channel = grpc.insecure_channel("target", ())
assert_type(grpc.channel_ready_future(channel).result(), None)

# Channel options supports list:
assert_type(grpc.insecure_channel("target", []), grpc.Channel)
assert_type(grpc.insecure_channel("target", [("a", "b")]), grpc.Channel)
assert_type(grpc.insecure_channel("target", [("a", "b"), ("c", "d")]), grpc.Channel)

# Client call details optionals:
call_details = grpc.ClientCallDetails()
assert_type(call_details.method, str)
assert_type(call_details.timeout, Optional[float])

# Call iterator
call_iter = cast(grpc._CallIterator[str], None)
for call in call_iter:
    assert_type(call, str)
assert_type(next(call_iter), str)



from collections.abc import Callable, Container
from types import TracebackType
from typing import Generic, Literal, Protocol, type_check_only
from typing_extensions import ParamSpec, Self, TypeAlias, TypeVarTuple, Unpack

from gevent._types import _Loop
from gevent.pool import Pool
from gevent.socket import socket as _GeventSocket
from greenlet import greenlet

_Ts = TypeVarTuple("_Ts")
_P = ParamSpec("_P")

@type_check_only
class _SpawnFunc(Protocol):
    def __call__(self, func: Callable[_P, object], /, *args: _P.args, **kwargs: _P.kwargs) -> greenlet: ...

_Spawner: TypeAlias = Pool | _SpawnFunc | int | Literal["default"] | None

class BaseServer(Generic[Unpack[_Ts]]):
    min_delay: float
    max_delay: float
    max_accept: int
    stop_timeout: float
    fatal_errors: Container[int]
    pool: Pool | None
    delay: float
    loop: _Loop
    family: int
    address: str | tuple[str, int]
    socket: _GeventSocket
    handle: Callable[[Unpack[_Ts]], object]
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        handle: Callable[[Unpack[_Ts]], object] | None = None,
        spawn: _Spawner = "default",
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, typ: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None, /) -> None: ...
    def set_listener(self, listener: _GeventSocket | tuple[str, int] | str) -> None: ...
    def set_spawn(self, spawn: _Spawner) -> None: ...
    def set_handle(self, handle: Callable[[Unpack[_Ts]], object]) -> None: ...
    def start_accepting(self) -> None: ...
    def stop_accepting(self) -> None: ...
    def do_handle(self, *args: Unpack[_Ts]) -> None: ...
    def do_close(self, *args: Unpack[_Ts]) -> None: ...
    def do_read(self) -> tuple[Unpack[_Ts]] | None: ...
    def full(self) -> bool: ...
    @property
    def server_host(self) -> str | None: ...
    @property
    def server_port(self) -> int | None: ...
    def init_socket(self) -> None: ...
    @property
    def started(self) -> bool: ...
    def start(self) -> None: ...
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def stop(self, timeout: float | None = None) -> None: ...
    def serve_forever(self, stop_timeout: float | None = None) -> None: ...
    def is_fatal_error(self, ex: BaseException) -> bool: ...

__all__ = ["BaseServer"]



from _typeshed import Incomplete

from braintree.braintree_gateway import BraintreeGateway

class ClientTokenGateway:
    gateway: BraintreeGateway
    config: Incomplete
    def __init__(self, gateway: BraintreeGateway) -> None: ...
    def generate(self, params: dict[str, Incomplete] | None = None) -> str: ...



# This is actually a C-extension module.
# Not all types defined in C are exported to Python.
# For example: `Pattern` and `Match` are not exported
# and are redefined in `regex.regex module.

from typing import Any, AnyStr, Generic, final
from typing_extensions import Self

from ._main import Match, Pattern

@final
class Splitter(Generic[AnyStr]):
    @property
    def pattern(self) -> Pattern[AnyStr]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> AnyStr | Any: ...
    def split(self) -> AnyStr | Any: ...

@final
class Scanner(Generic[AnyStr]):
    @property
    def pattern(self) -> Pattern[AnyStr]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> Match[AnyStr]: ...
    def match(self) -> Match[AnyStr] | None: ...
    def search(self) -> Match[AnyStr] | None: ...



import ast
import logging

from flake8_simplify.utils import Call

logger: logging.Logger

def get_sim115(node: Call) -> list[tuple[int, int, str]]: ...
def get_sim901(node: ast.Call) -> list[tuple[int, int, str]]: ...
def get_sim905(node: ast.Call) -> list[tuple[int, int, str]]: ...
def get_sim906(node: ast.Call) -> list[tuple[int, int, str]]: ...
def get_sim910(node: Call) -> list[tuple[int, int, str]]: ...
def get_sim911(node: ast.AST) -> list[tuple[int, int, str]]: ...



from _typeshed import Incomplete
from typing import Final, Literal

__version__: Final[str]

# NOTE: All the attributes in this module are Final
#       rl_config is the place to make changes
allowTableBoundsErrors: Final[int]
shapeChecking: Final[int]
defaultEncoding: Final[str]
defaultGraphicsFontName: Final[str]
pageCompression: Final[int]
useA85: Final[int]
defaultPageSize: Final[str]
defaultImageCaching: Final[int]
warnOnMissingFontGlyphs: Final[int]
verbose: Final[int]
showBoundary: Final[int]
emptyTableAction: Final[str]
invariant: Final[int]
eps_preview_transparent: Final[Incomplete]
eps_preview: Final[int]
eps_ttf_embed: Final[int]
eps_ttf_embed_uid: Final[int]
overlapAttachedSpace: Final[int]
longTableOptimize: Final[int]
autoConvertEncoding: Final[int]
_FUZZ: Final[float]
wrapA85: Final[int]
fsEncodings: Final[tuple[Literal["utf8"], Literal["cp1252"], Literal["cp430"]]]
odbc_driver: Final[str]
platypus_link_underline: Final[int]
canvas_basefontname: Final[str]
allowShortTableRows: Final[int]
imageReaderFlags: Final[int]
paraFontSizeHeightOffset: Final[int]
canvas_baseColor: Final[Incomplete]
ignoreContainerActions: Final[int]
ttfAsciiReadable: Final[int]
pdfMultiLine: Final[int]
pdfComments: Final[int]
debug: Final[int]
listWrapOnFakeWidth: Final[int]
underlineWidth: Final[str]
underlineOffset: Final[str]
underlineGap: Final[str]
strikeWidth: Final[str]
strikeOffset: Final[str]
strikeGap: Final[str]
decimalSymbol: Final[str]
errorOnDuplicatePageLabelPage: Final[int]
autoGenerateMissingTTFName: Final[int]
allowTTFSubsetting: Final[list[str]]
spaceShrinkage: Final[float]
hyphenationLang: Final[str]
uriWasteReduce: Final[int]
embeddedHyphenation: Final[int]
hyphenationMinWordLength: Final[int]
reserveTTFNotdef: Final[int]
documentLang: Final[Incomplete]
encryptionStrength: Final[int]
trustedHosts: Final[Incomplete]
trustedSchemes: Final[list[str]]
renderPMBackend: Final[str]
xmlParser: Final[str]
textPaths: Final[str]
toColorCanUse: Final[str]
defCWRF: Final[float]
unShapedFontGlob: list[str] | None
T1SearchPath: Final[tuple[str, ...]]
TTFSearchPath: Final[tuple[str, ...]]
CMapSearchPath: Final[tuple[str, ...]]

__all__ = (
    "allowTableBoundsErrors",
    "shapeChecking",
    "defaultEncoding",
    "defaultGraphicsFontName",
    "pageCompression",
    "useA85",
    "defaultPageSize",
    "defaultImageCaching",
    "warnOnMissingFontGlyphs",
    "verbose",
    "showBoundary",
    "emptyTableAction",
    "invariant",
    "eps_preview_transparent",
    "eps_preview",
    "eps_ttf_embed",
    "eps_ttf_embed_uid",
    "overlapAttachedSpace",
    "longTableOptimize",
    "autoConvertEncoding",
    "_FUZZ",
    "wrapA85",
    "fsEncodings",
    "odbc_driver",
    "platypus_link_underline",
    "canvas_basefontname",
    "allowShortTableRows",
    "imageReaderFlags",
    "paraFontSizeHeightOffset",
    "canvas_baseColor",
    "ignoreContainerActions",
    "ttfAsciiReadable",
    "pdfMultiLine",
    "pdfComments",
    "debug",
    "listWrapOnFakeWidth",
    "T1SearchPath",
    "TTFSearchPath",
    "CMapSearchPath",
    "decimalSymbol",
    "errorOnDuplicatePageLabelPage",
    "autoGenerateMissingTTFName",
    "allowTTFSubsetting",
    "spaceShrinkage",
    "underlineWidth",
    "underlineOffset",
    "underlineGap",
    "strikeWidth",
    "strikeOffset",
    "strikeGap",
    "hyphenationLang",
    "uriWasteReduce",
    "embeddedHyphenation",
    "hyphenationMinWordLength",
    "reserveTTFNotdef",
    "documentLang",
    "encryptionStrength",
    "trustedHosts",
    "trustedSchemes",
    "renderPMBackend",
    "xmlParser",
    "textPaths",
    "toColorCanUse",
    "defCWRF",
    "unShapedFontGlob",
)



from collections import UserString
from typing import Any, Literal

from pandas import Series

class VarType(UserString):
    allowed: tuple[str, ...]
    def __init__(self, data: str) -> None: ...
    def __eq__(self, other: str) -> bool: ...  # type: ignore[override]

def variable_type(
    vector: Series[Any], boolean_type: Literal["numeric", "categorical", "boolean"] = "numeric", strict_boolean: bool = False
) -> VarType: ...
def categorical_order(vector: Series[Any], order: list[Any] | None = None) -> list[Any]: ...



from collections.abc import Callable, Generator, Iterable, Iterator
from typing import Literal

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "dfs_edges",
    "dfs_tree",
    "dfs_predecessors",
    "dfs_successors",
    "dfs_preorder_nodes",
    "dfs_postorder_nodes",
    "dfs_labeled_edges",
]

@_dispatchable
def dfs_edges(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> Generator[tuple[_Node, _Node]]: ...
@_dispatchable
def dfs_tree(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> DiGraph[_Node]: ...
@_dispatchable
def dfs_predecessors(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> dict[_Node, _Node]: ...
@_dispatchable
def dfs_successors(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> dict[_Node, list[_Node]]: ...
@_dispatchable
def dfs_postorder_nodes(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> Generator[_Node]: ...
@_dispatchable
def dfs_preorder_nodes(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> Generator[_Node]: ...
@_dispatchable
def dfs_labeled_edges(
    G: Graph[_Node],
    source: _Node | None = None,
    depth_limit: int | None = None,
    *,
    sort_neighbors: Callable[[Iterator[_Node]], Iterable[_Node]] | None = None,
) -> Generator[tuple[_Node, _Node, Literal["forward", "nontree", "reverse", "reverse-depth_limit"]]]: ...



from logging import Logger
from typing import Final

log: Logger
CONF_PATH: Final = "/var/elasticbeanstalk/xray/environment.conf"
SERVICE_NAME: Final = "elastic_beanstalk"
ORIGIN: Final = "AWS::ElasticBeanstalk::Environment"

def initialize() -> None: ...



from collections.abc import Callable, Iterable, Sequence
from typing import Any, Generic, TypeVar
from typing_extensions import Self, disjoint_base

from gevent._types import _AddrinfoResult, _Loop, _NameinfoResult, _SockAddr

_T = TypeVar("_T")

class ares_host_result(tuple[str, list[str], list[str]]):
    family: int
    def __new__(cls, family: int, iterable: Iterable[Any]) -> Self: ...

@disjoint_base
class Result(Generic[_T]):
    exception: BaseException | None
    value: _T | None
    def __init__(self, value: _T | None = None, exception: BaseException | None = None) -> None: ...
    def get(self) -> Any | None: ...
    def successful(self) -> bool: ...

@disjoint_base
class channel:
    @property
    def loop(self) -> _Loop: ...
    def __init__(
        self,
        loop: _Loop,
        flags: str | int | None = None,
        timeout: str | float | None = None,
        tries: str | int | None = None,
        ndots: str | int | None = None,
        udp_port: str | int | None = None,
        tcp_port: str | int | None = None,
        servers: Sequence[str] | str | None = None,
    ) -> None: ...
    def destroy(self) -> None: ...
    def getaddrinfo(
        self,
        callback: Callable[[Result[_AddrinfoResult]], object],
        name: str,
        service: str | None,
        family: int = 0,
        type: int = 0,
        proto: int = 0,
        flags: int = 0,
    ) -> None: ...
    def gethostbyaddr(self, callback: Callable[[Result[ares_host_result]], object], addr: str) -> Any: ...
    def gethostbyname(self, callback: Callable[[Result[ares_host_result]], object], name: str, family: int = 2) -> None: ...
    def getnameinfo(self, callback: Callable[[Result[_NameinfoResult]], object], sockaddr: _SockAddr, flags: int) -> None: ...
    def set_servers(self, servers: Sequence[str] | str | None = None) -> None: ...

__all__ = ["channel"]



from .device_code import DEVICE_CODE_GRANT_TYPE as DEVICE_CODE_GRANT_TYPE, DeviceCodeGrant as DeviceCodeGrant
from .endpoint import DeviceAuthorizationEndpoint as DeviceAuthorizationEndpoint
from .errors import (
    AuthorizationPendingError as AuthorizationPendingError,
    ExpiredTokenError as ExpiredTokenError,
    SlowDownError as SlowDownError,
)
from .models import DeviceCredentialDict as DeviceCredentialDict, DeviceCredentialMixin as DeviceCredentialMixin

__all__ = [
    "DeviceAuthorizationEndpoint",
    "DeviceCodeGrant",
    "DEVICE_CODE_GRANT_TYPE",
    "DeviceCredentialMixin",
    "DeviceCredentialDict",
    "AuthorizationPendingError",
    "SlowDownError",
    "ExpiredTokenError",
]



from gunicorn.config import Config

PLATFORM: str
IS_CYGWIN: bool

class WorkerTmp:
    def __init__(self, cfg: Config) -> None: ...
    def notify(self) -> None: ...
    def last_update(self) -> float: ...
    def fileno(self) -> int: ...
    def close(self) -> None: ...



class AuthlibDeprecationWarning(DeprecationWarning): ...

def deprecate(
    message: str, version: str | None = None, link_uid: str | None = None, link_file: str | None = None, stacklevel: int = 3
) -> None: ...



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
import sys
from typing import Any, List, Optional, Tuple, Union

import attrs
import cattrs

from . import types as lsp_types

LSPAny = lsp_types.LSPAny
OptionalPrimitive = Optional[Union[bool, int, str, float]]

# Flag to ensure we only resolve forward references once.
_resolved_forward_references = False


def _resolve_forward_references() -> None:
    """Resolve forward references for faster processing with cattrs."""
    global _resolved_forward_references
    if not _resolved_forward_references:

        def _filter(p: Tuple[str, object]) -> bool:
            return isinstance(p[1], type) and attrs.has(p[1])

        # Creating a concrete list here because `resolve_types` mutates the provided map.
        items = list(filter(_filter, lsp_types.ALL_TYPES_MAP.items()))
        for _, value in items:
            if isinstance(value, type):
                attrs.resolve_types(value, lsp_types.ALL_TYPES_MAP, {})  # type: ignore
        _resolved_forward_references = True


def register_hooks(converter: cattrs.Converter) -> cattrs.Converter:
    _resolve_forward_references()
    converter = _register_capabilities_hooks(converter)
    converter = _register_required_structure_hooks(converter)
    return _register_custom_property_hooks(converter)


def _register_capabilities_hooks(converter: cattrs.Converter) -> cattrs.Converter:
    def _text_document_sync_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.TextDocumentSyncOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.TextDocumentSyncOptions)

    def _notebook_document_sync_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[
            lsp_types.NotebookDocumentSyncRegistrationOptions,
            lsp_types.NotebookDocumentSyncOptions,
        ]
    ]:
        if object_ is None:
            return None
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.NotebookDocumentSyncRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.NotebookDocumentSyncOptions)

    def _hover_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.HoverOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.HoverOptions)

    def _declaration_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.DeclarationRegistrationOptions,
        lsp_types.DeclarationOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.DeclarationRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.DeclarationOptions)

    def _definition_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.DefinitionOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.DefinitionOptions)

    def _type_definition_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.TypeDefinitionRegistrationOptions,
        lsp_types.TypeDefinitionOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.TypeDefinitionRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.TypeDefinitionOptions)

    def _implementation_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.ImplementationRegistrationOptions,
        lsp_types.ImplementationOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.ImplementationRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.ImplementationOptions)

    def _references_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.ReferenceOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.ReferenceOptions)

    def _position_encoding_hook(
        object_: Union[lsp_types.PositionEncodingKind, OptionalPrimitive], _: type
    ) -> Union[lsp_types.PositionEncodingKind, OptionalPrimitive]:
        return object_

    def _document_highlight_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.DocumentHighlightOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.DocumentHighlightOptions)

    def _document_symbol_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.DocumentSymbolOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.DocumentSymbolOptions)

    def _code_action_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.CodeActionOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.CodeActionOptions)

    def _color_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.DocumentColorRegistrationOptions,
        lsp_types.DocumentColorOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.DocumentColorRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.DocumentColorOptions)

    def _workspace_symbol_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.WorkspaceSymbolOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.WorkspaceSymbolOptions)

    def _document_formatting_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.DocumentFormattingOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.DocumentFormattingOptions)

    def _document_range_formatting_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.DocumentRangeFormattingOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.DocumentRangeFormattingOptions)

    def _rename_provider_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.RenameOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.RenameOptions)

    def _folding_range_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.FoldingRangeRegistrationOptions,
        lsp_types.FoldingRangeOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.FoldingRangeRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.FoldingRangeOptions)

    def _selection_range_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.SelectionRangeRegistrationOptions,
        lsp_types.SelectionRangeOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.SelectionRangeRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.SelectionRangeOptions)

    def _call_hierarchy_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.CallHierarchyRegistrationOptions,
        lsp_types.CallHierarchyOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.CallHierarchyRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.CallHierarchyOptions)

    def _linked_editing_range_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.LinkedEditingRangeRegistrationOptions,
        lsp_types.LinkedEditingRangeOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.LinkedEditingRangeRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.LinkedEditingRangeOptions)

    def _semantic_tokens_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.SemanticTokensRegistrationOptions,
        lsp_types.SemanticTokensOptions,
    ]:
        if object_ is None:
            return None
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.SemanticTokensRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.SemanticTokensOptions)

    def _moniker_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.MonikerRegistrationOptions,
        lsp_types.MonikerOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(object_, lsp_types.MonikerRegistrationOptions)
        else:
            return converter.structure(object_, lsp_types.MonikerOptions)

    def _type_hierarchy_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.TypeHierarchyRegistrationOptions,
        lsp_types.TypeHierarchyOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.TypeHierarchyRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.TypeHierarchyOptions)

    def _inline_value_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.InlineValueRegistrationOptions,
        lsp_types.InlineValueOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(
                object_, lsp_types.InlineValueRegistrationOptions
            )
        else:
            return converter.structure(object_, lsp_types.InlineValueOptions)

    def _inlay_hint_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.InlayHintRegistrationOptions,
        lsp_types.InlayHintOptions,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if "id" in object_:
            return converter.structure(object_, lsp_types.InlayHintRegistrationOptions)
        else:
            return converter.structure(object_, lsp_types.InlayHintOptions)

    def _inlay_hint_label_part_hook(
        object_: Any, _: type
    ) -> Union[str, List[lsp_types.InlayHintLabelPart]]:
        if isinstance(object_, str):
            return object_

        return [
            converter.structure(item, lsp_types.InlayHintLabelPart) for item in object_
        ]

    def _diagnostic_provider_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.DiagnosticRegistrationOptions,
        lsp_types.DiagnosticOptions,
    ]:
        if object_ is None:
            return None
        if "id" in object_:
            return converter.structure(object_, lsp_types.DiagnosticRegistrationOptions)
        else:
            return converter.structure(object_, lsp_types.DiagnosticOptions)

    def _save_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.SaveOptions]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.SaveOptions)

    def _code_action_hook(
        object_: Any, _: type
    ) -> Union[lsp_types.Command, lsp_types.CodeAction]:
        if "command" in object_:
            return converter.structure(object_, lsp_types.Command)
        else:
            return converter.structure(object_, lsp_types.CodeAction)

    def _completion_list_hook(
        object_: Any, _: type
    ) -> Optional[Union[lsp_types.CompletionList, List[lsp_types.CompletionItem]]]:
        if object_ is None:
            return None
        if isinstance(object_, list):
            return [
                converter.structure(item, lsp_types.CompletionItem) for item in object_
            ]
        else:
            return converter.structure(object_, lsp_types.CompletionList)

    def _location_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[
            lsp_types.Location,
            List[lsp_types.Location],
            List[lsp_types.LocationLink],
        ]
    ]:
        if object_ is None:
            return None
        if isinstance(object_, list):
            if len(object_) == 0:
                return []
            if "targetUri" in object_[0]:
                return [
                    converter.structure(item, lsp_types.LocationLink)
                    for item in object_
                ]
            else:
                return [
                    converter.structure(item, lsp_types.Location) for item in object_
                ]
        else:
            return converter.structure(object_, lsp_types.Location)

    def _symbol_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[List[lsp_types.DocumentSymbol], List[lsp_types.SymbolInformation]]
    ]:
        if object_ is None:
            return None
        if isinstance(object_, list):
            if len(object_) == 0:
                return []
            if "location" in object_[0]:
                return [
                    converter.structure(item, lsp_types.SymbolInformation)
                    for item in object_
                ]
            else:
                return [
                    converter.structure(item, lsp_types.DocumentSymbol)
                    for item in object_
                ]
        else:
            return None

    def _markup_content_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[
            OptionalPrimitive,
            lsp_types.MarkupContent,
            lsp_types.MarkedString_Type1,
            List[Union[OptionalPrimitive, lsp_types.MarkedString_Type1]],
        ]
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        if isinstance(object_, list):
            return [
                (
                    item
                    if isinstance(item, (bool, int, str, float))
                    else converter.structure(item, lsp_types.MarkedString_Type1)
                )
                for item in object_
            ]
        if "kind" in object_:
            return converter.structure(object_, lsp_types.MarkupContent)
        else:
            return converter.structure(object_, lsp_types.MarkedString_Type1)

    def _document_edit_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[
            lsp_types.TextDocumentEdit,
            lsp_types.CreateFile,
            lsp_types.RenameFile,
            lsp_types.DeleteFile,
        ]
    ]:
        if object_ is None:
            return None
        if "kind" in object_:
            if object_["kind"] == "create":
                return converter.structure(object_, lsp_types.CreateFile)
            elif object_["kind"] == "rename":
                return converter.structure(object_, lsp_types.RenameFile)
            elif object_["kind"] == "delete":
                return converter.structure(object_, lsp_types.DeleteFile)
            else:
                raise ValueError("Unknown edit kind: ", object_)
        else:
            return converter.structure(object_, lsp_types.TextDocumentEdit)

    def _semantic_tokens_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.SemanticTokensOptionsFullType1]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.SemanticTokensOptionsFullType1)

    def _semantic_tokens_capabilities_hook(
        object_: Any, _: type
    ) -> Union[
        OptionalPrimitive,
        lsp_types.SemanticTokensClientCapabilitiesRequestsTypeFullType1,
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(
            object_, lsp_types.SemanticTokensClientCapabilitiesRequestsTypeFullType1
        )

    def _code_action_kind_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.CodeActionKind]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.CodeActionKind)

    def _position_encoding_kind_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.PositionEncodingKind]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.PositionEncodingKind)

    def _folding_range_kind_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.FoldingRangeKind]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.FoldingRangeKind)

    def _semantic_token_types_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.SemanticTokenTypes]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.SemanticTokenTypes)

    def _semantic_token_modifiers_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.SemanticTokenModifiers]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.SemanticTokenModifiers)

    def _watch_kind_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.WatchKind]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.WatchKind)

    def _notebook_sync_option_selector_hook(
        object_: Any, _: type
    ) -> Union[
        lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType1,
        lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType2,
    ]:
        if "notebook" in object_:
            return converter.structure(
                object_, lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType1
            )
        else:
            return converter.structure(
                object_, lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType2
            )

    def _semantic_token_registration_options_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[OptionalPrimitive, lsp_types.SemanticTokensRegistrationOptionsFullType1]
    ]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(
            object_, lsp_types.SemanticTokensRegistrationOptionsFullType1
        )

    def _inline_completion_provider_hook(
        object_: Any, _: type
    ) -> Optional[Union[OptionalPrimitive, lsp_types.InlineCompletionOptions]]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.InlineCompletionOptions)

    def _inline_completion_list_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[lsp_types.InlineCompletionList, List[lsp_types.InlineCompletionItem]]
    ]:
        if object_ is None:
            return None
        if isinstance(object_, list):
            return [
                converter.structure(item, lsp_types.InlineCompletionItem)
                for item in object_
            ]
        return converter.structure(object_, lsp_types.InlineCompletionList)

    def _string_value_hook(
        object_: Any, _: type
    ) -> Union[OptionalPrimitive, lsp_types.StringValue]:
        if object_ is None:
            return None
        if isinstance(object_, (bool, int, str, float)):
            return object_
        return converter.structure(object_, lsp_types.StringValue)

    def _symbol_list_hook(
        object_: Any, _: type
    ) -> Optional[
        Union[List[lsp_types.SymbolInformation], List[lsp_types.WorkspaceSymbol]]
    ]:
        if object_ is None:
            return None
        assert isinstance(object_, list)
        if len(object_) == 0:
            return []
        if "deprecated" in object_[0]:
            return [
                converter.structure(item, lsp_types.SymbolInformation)
                for item in object_
            ]
        elif ("data" in object_[0]) or ("range" not in object_[0]["location"]):
            return [
                converter.structure(item, lsp_types.WorkspaceSymbol) for item in object_
            ]

        return [
            converter.structure(item, lsp_types.SymbolInformation) for item in object_
        ]

    def _notebook_sync_registration_option_selector_hook(
        object_: Any, _: type
    ) -> Union[
        lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType1,
        lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType2,
    ]:
        if "notebook" in object_:
            return converter.structure(
                object_,
                lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType1,
            )
        else:
            return converter.structure(
                object_,
                lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType2,
            )

    structure_hooks = [
        (
            Optional[
                Union[lsp_types.TextDocumentSyncOptions, lsp_types.TextDocumentSyncKind]
            ],
            _text_document_sync_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.NotebookDocumentSyncOptions,
                    lsp_types.NotebookDocumentSyncRegistrationOptions,
                ]
            ],
            _notebook_document_sync_hook,
        ),
        (Optional[Union[bool, lsp_types.HoverOptions]], _hover_provider_hook),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.DeclarationOptions,
                    lsp_types.DeclarationRegistrationOptions,
                ]
            ],
            _declaration_provider_hook,
        ),
        (Optional[Union[bool, lsp_types.DefinitionOptions]], _definition_provider_hook),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.TypeDefinitionOptions,
                    lsp_types.TypeDefinitionRegistrationOptions,
                ]
            ],
            _type_definition_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.ImplementationOptions,
                    lsp_types.ImplementationRegistrationOptions,
                ]
            ],
            _implementation_provider_hook,
        ),
        (Optional[Union[bool, lsp_types.ReferenceOptions]], _references_provider_hook),
        (
            Optional[Union[bool, lsp_types.DocumentHighlightOptions]],
            _document_highlight_provider_hook,
        ),
        (
            Optional[Union[bool, lsp_types.DocumentSymbolOptions]],
            _document_symbol_provider_hook,
        ),
        (
            Optional[Union[bool, lsp_types.CodeActionOptions]],
            _code_action_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.DocumentColorOptions,
                    lsp_types.DocumentColorRegistrationOptions,
                ]
            ],
            _color_provider_hook,
        ),
        (
            Optional[Union[bool, lsp_types.WorkspaceSymbolOptions]],
            _workspace_symbol_provider_hook,
        ),
        (
            Optional[Union[bool, lsp_types.DocumentFormattingOptions]],
            _document_formatting_provider_hook,
        ),
        (
            Optional[Union[bool, lsp_types.DocumentRangeFormattingOptions]],
            _document_range_formatting_provider_hook,
        ),
        (Optional[Union[bool, lsp_types.RenameOptions]], _rename_provider_hook),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.FoldingRangeOptions,
                    lsp_types.FoldingRangeRegistrationOptions,
                ]
            ],
            _folding_range_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.SelectionRangeOptions,
                    lsp_types.SelectionRangeRegistrationOptions,
                ]
            ],
            _selection_range_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.CallHierarchyOptions,
                    lsp_types.CallHierarchyRegistrationOptions,
                ]
            ],
            _call_hierarchy_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.LinkedEditingRangeOptions,
                    lsp_types.LinkedEditingRangeRegistrationOptions,
                ]
            ],
            _linked_editing_range_provider_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.SemanticTokensOptions,
                    lsp_types.SemanticTokensRegistrationOptions,
                ]
            ],
            _semantic_tokens_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool, lsp_types.MonikerOptions, lsp_types.MonikerRegistrationOptions
                ]
            ],
            _moniker_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.TypeHierarchyOptions,
                    lsp_types.TypeHierarchyRegistrationOptions,
                ]
            ],
            _type_hierarchy_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.InlineValueOptions,
                    lsp_types.InlineValueRegistrationOptions,
                ]
            ],
            _inline_value_provider_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.InlayHintOptions,
                    lsp_types.InlayHintRegistrationOptions,
                ]
            ],
            _inlay_hint_provider_hook,
        ),
        (
            Union[str, List[lsp_types.InlayHintLabelPart]],
            _inlay_hint_label_part_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.DiagnosticOptions, lsp_types.DiagnosticRegistrationOptions
                ]
            ],
            _diagnostic_provider_hook,
        ),
        (
            Optional[Union[lsp_types.SaveOptions, bool]],
            _save_hook,
        ),
        (
            Union[lsp_types.Command, lsp_types.CodeAction],
            _code_action_hook,
        ),
        (
            Optional[Union[List[lsp_types.CompletionItem], lsp_types.CompletionList]],
            _completion_list_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.Location,
                    List[lsp_types.Location],
                    List[lsp_types.LocationLink],
                ]
            ],
            _location_hook,
        ),
        (
            Optional[
                Union[List[lsp_types.SymbolInformation], List[lsp_types.DocumentSymbol]]
            ],
            _symbol_hook,
        ),
        (
            Union[
                lsp_types.MarkupContent,
                str,
                lsp_types.MarkedString_Type1,
                List[Union[str, lsp_types.MarkedString_Type1]],
            ],
            _markup_content_hook,
        ),
        (
            Union[
                lsp_types.TextDocumentEdit,
                lsp_types.CreateFile,
                lsp_types.RenameFile,
                lsp_types.DeleteFile,
            ],
            _document_edit_hook,
        ),
        (
            Optional[Union[bool, lsp_types.SemanticTokensOptionsFullType1]],
            _semantic_tokens_hook,
        ),
        (
            Optional[
                Union[
                    bool,
                    lsp_types.SemanticTokensClientCapabilitiesRequestsTypeFullType1,
                ]
            ],
            _semantic_tokens_capabilities_hook,
        ),
        (
            Optional[Union[str, lsp_types.MarkupContent]],
            _markup_content_hook,
        ),
        (
            Optional[Union[lsp_types.CodeActionKind, str]],
            _code_action_kind_hook,
        ),
        (
            Union[lsp_types.CodeActionKind, str],
            _code_action_kind_hook,
        ),
        (
            Union[lsp_types.PositionEncodingKind, str],
            _position_encoding_kind_hook,
        ),
        (
            Optional[Union[lsp_types.FoldingRangeKind, str]],
            _folding_range_kind_hook,
        ),
        (
            Union[lsp_types.FoldingRangeKind, str],
            _folding_range_kind_hook,
        ),
        (
            Union[lsp_types.SemanticTokenTypes, str],
            _semantic_token_types_hook,
        ),
        (
            Optional[Union[lsp_types.SemanticTokenTypes, str]],
            _semantic_token_types_hook,
        ),
        (
            Union[lsp_types.SemanticTokenModifiers, str],
            _semantic_token_modifiers_hook,
        ),
        (
            Optional[Union[lsp_types.SemanticTokenModifiers, str]],
            _semantic_token_modifiers_hook,
        ),
        (
            Union[lsp_types.WatchKind, int],
            _watch_kind_hook,
        ),
        (
            Optional[Union[lsp_types.WatchKind, int]],
            _watch_kind_hook,
        ),
        (
            Union[
                lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType1,
                lsp_types.NotebookDocumentSyncOptionsNotebookSelectorType2,
            ],
            _notebook_sync_option_selector_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.PositionEncodingKind,
                    str,
                ]
            ],
            _position_encoding_hook,
        ),
        (
            Optional[Union[bool, lsp_types.SemanticTokensRegistrationOptionsFullType1]],
            _semantic_token_registration_options_hook,
        ),
        (
            Optional[Union[bool, lsp_types.InlineCompletionOptions]],
            _inline_completion_provider_hook,
        ),
        (
            Optional[
                Union[
                    lsp_types.InlineCompletionList, List[lsp_types.InlineCompletionItem]
                ]
            ],
            _inline_completion_list_hook,
        ),
        (
            Union[str, lsp_types.StringValue],
            _string_value_hook,
        ),
        (
            Optional[
                Union[
                    List[lsp_types.SymbolInformation], List[lsp_types.WorkspaceSymbol]
                ]
            ],
            _symbol_list_hook,
        ),
        (
            Union[
                lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType1,
                lsp_types.NotebookDocumentSyncRegistrationOptionsNotebookSelectorType2,
            ],
            _notebook_sync_registration_option_selector_hook,
        ),
    ]
    for type_, hook in structure_hooks:
        converter.register_structure_hook(type_, hook)
    return converter


def _register_required_structure_hooks(
    converter: cattrs.Converter,
) -> cattrs.Converter:
    def _lsp_object_hook(object_: Any, type_: type) -> Any:
        return object_

    def _parameter_information_label_hook(
        object_: Any, type: type
    ) -> Union[str, Tuple[int, int]]:
        if isinstance(object_, str):
            return object_
        else:
            return (int(object_[0]), int(object_[1]))

    def _text_document_filter_hook(
        object_: Any, _: type
    ) -> Union[
        str,
        lsp_types.TextDocumentFilter_Type1,
        lsp_types.TextDocumentFilter_Type2,
        lsp_types.TextDocumentFilter_Type3,
        lsp_types.NotebookCellTextDocumentFilter,
    ]:
        if isinstance(object_, str):
            return str(object_)
        elif "notebook" in object_:
            return converter.structure(
                object_, lsp_types.NotebookCellTextDocumentFilter
            )
        elif "language" in object_:
            return converter.structure(object_, lsp_types.TextDocumentFilter_Type1)
        elif "scheme" in object_:
            return converter.structure(object_, lsp_types.TextDocumentFilter_Type2)
        else:
            return converter.structure(object_, lsp_types.TextDocumentFilter_Type3)

    def _notebook_filter_hook(
        object_: Any, _: type
    ) -> Union[
        str,
        lsp_types.NotebookDocumentFilter_Type1,
        lsp_types.NotebookDocumentFilter_Type2,
        lsp_types.NotebookDocumentFilter_Type3,
    ]:
        if isinstance(object_, str):
            return str(object_)
        elif "notebookType" in object_:
            return converter.structure(object_, lsp_types.NotebookDocumentFilter_Type1)
        elif "scheme" in object_:
            return converter.structure(object_, lsp_types.NotebookDocumentFilter_Type2)
        else:
            return converter.structure(object_, lsp_types.NotebookDocumentFilter_Type3)

    # TODO: Remove the ignore after this issue with attrs is addressed in either attrs or mypy
    NotebookSelectorItem = attrs.fields(
        lsp_types.NotebookCellTextDocumentFilter
    ).notebook.type
    STRUCTURE_HOOKS = [
        (type(None), lambda object_, _type: object_),
        (Optional[Union[int, str]], lambda object_, _type: object_),
        (Union[int, str], lambda object_, _type: object_),
        (lsp_types.LSPAny, _lsp_object_hook),
        (Optional[Union[str, bool]], lambda object_, _type: object_),
        (Optional[Union[bool, Any]], lambda object_, _type: object_),
        (
            Union[
                lsp_types.TextDocumentFilter_Type1,
                lsp_types.TextDocumentFilter_Type2,
                lsp_types.TextDocumentFilter_Type3,
                lsp_types.NotebookCellTextDocumentFilter,
            ],
            _text_document_filter_hook,
        ),
        (lsp_types.DocumentFilter, _text_document_filter_hook),
        (
            Union[
                str,
                lsp_types.NotebookDocumentFilter_Type1,
                lsp_types.NotebookDocumentFilter_Type2,
                lsp_types.NotebookDocumentFilter_Type3,
            ],
            _notebook_filter_hook,
        ),
        (NotebookSelectorItem, _notebook_filter_hook),
        (
            Union[lsp_types.LSPObject, List["LSPAny"], str, int, float, bool, None],
            _lsp_object_hook,
        ),
        (
            Union[
                lsp_types.LSPObject, List[lsp_types.LSPAny], str, int, float, bool, None
            ],
            _lsp_object_hook,
        ),
        (
            Union[str, Tuple[int, int]],
            _parameter_information_label_hook,
        ),
        (lsp_types.LSPObject, _lsp_object_hook),
    ]

    if sys.version_info > (3, 8):
        STRUCTURE_HOOKS += [
            (
                Union[
                    lsp_types.LSPObject,
                    List[
                        Union[
                            lsp_types.LSPObject,
                            List["LSPAny"],
                            str,
                            int,
                            float,
                            bool,
                            None,
                        ]
                    ],
                    str,
                    int,
                    float,
                    bool,
                    None,
                ],
                _lsp_object_hook,
            )
        ]

    for type_, hook in STRUCTURE_HOOKS:
        converter.register_structure_hook(type_, hook)

    return converter


def _register_custom_property_hooks(converter: cattrs.Converter) -> cattrs.Converter:
    def _to_camel_case(name: str) -> str:
        # TODO: when min Python becomes >= 3.9, then update this to:
        # `return name.removesuffix("_")`.
        new_name = name[:-1] if name.endswith("_") else name
        parts = new_name.split("_")
        return parts[0] + "".join(p.title() for p in parts[1:])

    def _omit(cls: type, prop: str) -> bool:
        special = lsp_types.is_special_property(cls, prop)
        return not special

    def _with_custom_unstructure(cls: type) -> Any:
        attributes = {
            a.name: cattrs.gen.override(
                rename=_to_camel_case(a.name),
                omit_if_default=_omit(cls, a.name),
            )
            for a in attrs.fields(cls)
        }
        return cattrs.gen.make_dict_unstructure_fn(cls, converter, **attributes)

    def _with_custom_structure(cls: type) -> Any:
        attributes = {
            a.name: cattrs.gen.override(
                rename=_to_camel_case(a.name),
                omit_if_default=_omit(cls, a.name),
            )
            for a in attrs.fields(cls)
        }
        return cattrs.gen.make_dict_structure_fn(cls, converter, **attributes)

    converter.register_unstructure_hook_factory(attrs.has, _with_custom_unstructure)
    converter.register_structure_hook_factory(attrs.has, _with_custom_structure)
    return converter



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Blacklists:
    url: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get(self, aud: str | None = None) -> list[dict[str, str]]: ...
    async def get_async(self, aud: str | None = None) -> list[dict[str, str]]: ...
    def create(self, jti: str, aud: str | None = None) -> dict[str, str]: ...
    async def create_async(self, jti: str, aud: str | None = None) -> dict[str, str]: ...



import argparse
from logging import Logger

from ..formatting.base import BaseFormatter
from .finder import LoadedPlugin

LOG: Logger

def make(reporters: dict[str, LoadedPlugin], options: argparse.Namespace) -> BaseFormatter: ...



from collections.abc import Iterable
from typing import Any, Final, Literal, overload

import numpy as np
from numpy.typing import ArrayLike, NDArray
from shapely import Geometry

from .array import _Array1D, _Array2D

PREDICATES: Final[set[str | None]]

class SpatialIndex:
    geometries: NDArray[np.object_]
    def __init__(self, geometry: NDArray[np.object_]) -> None: ...
    @property
    def valid_query_predicates(self) -> set[str | None]: ...
    @overload
    def query(
        self,
        geometry: Geometry | ArrayLike,
        predicate: str | None = None,
        sort: bool = False,
        distance: float | ArrayLike | None = None,
        output_format: Literal["indices"] = "indices",
    ) -> NDArray[np.int64]: ...
    @overload
    def query(
        self,
        geometry: Geometry | ArrayLike,
        predicate: str | None = None,
        sort: bool = False,
        distance: float | ArrayLike | None = None,
        *,
        output_format: Literal["dense"],
    ) -> NDArray[np.bool_]: ...
    @overload
    def query(
        self,
        geometry: Geometry | ArrayLike,
        predicate: str | None = None,
        sort: bool = False,
        distance: float | ArrayLike | None = None,
        *,
        output_format: Literal["sparse"],
    ) -> Any: ...  # returns scipy coo_array but we don't depend on scipy
    @overload
    def nearest(
        self,
        geometry,
        return_all: bool = True,
        max_distance: float | None = None,
        return_distance: Literal[False] = False,
        exclusive: bool = False,
    ) -> _Array2D[np.int64]: ...
    @overload
    def nearest(
        self,
        geometry,
        return_all: bool = True,
        max_distance: float | None = None,
        *,
        return_distance: Literal[True],
        exclusive: bool = False,
    ) -> tuple[_Array2D[np.int64], _Array1D[np.float64]]: ...
    @overload
    def nearest(
        self,
        geometry,
        return_all: bool = True,
        max_distance: float | None = None,
        return_distance: bool = False,
        exclusive: bool = False,
    ) -> _Array2D[np.int64] | tuple[_Array2D[np.int64], _Array1D[np.float64]]: ...
    def intersection(self, coordinates: Iterable[float]) -> _Array1D[np.int64]: ...
    @property
    def size(self) -> int: ...
    @property
    def is_empty(self) -> bool: ...
    def __len__(self) -> int: ...



from _socket import SocketType
from logging import Logger

LOGGER: Logger

_SUPPORTED_TCP_OPTIONS: dict[str, int]

def socket_requires_keepalive(tcp_options: dict[str, int]) -> bool: ...
def set_sock_opts(tcp_options: dict[str, int] | None, sock: SocketType) -> None: ...



from typing import ClassVar

from .common import InfoExtractor

class RtmpIE(InfoExtractor):
    IE_DESC: ClassVar[bool]

class MmsIE(InfoExtractor):
    IE_DESC: ClassVar[bool]

class ViewSourceIE(InfoExtractor):
    IE_DESC: ClassVar[bool]



from typing import Final

num_glyphs: Final = 154
X_cursor: Final = 0
arrow: Final = 2
based_arrow_down: Final = 4
based_arrow_up: Final = 6
boat: Final = 8
bogosity: Final = 10
bottom_left_corner: Final = 12
bottom_right_corner: Final = 14
bottom_side: Final = 16
bottom_tee: Final = 18
box_spiral: Final = 20
center_ptr: Final = 22
circle: Final = 24
clock: Final = 26
coffee_mug: Final = 28
cross: Final = 30
cross_reverse: Final = 32
crosshair: Final = 34
diamond_cross: Final = 36
dot: Final = 38
dotbox: Final = 40
double_arrow: Final = 42
draft_large: Final = 44
draft_small: Final = 46
draped_box: Final = 48
exchange: Final = 50
fleur: Final = 52
gobbler: Final = 54
gumby: Final = 56
hand1: Final = 58
hand2: Final = 60
heart: Final = 62
icon: Final = 64
iron_cross: Final = 66
left_ptr: Final = 68
left_side: Final = 70
left_tee: Final = 72
leftbutton: Final = 74
ll_angle: Final = 76
lr_angle: Final = 78
man: Final = 80
middlebutton: Final = 82
mouse: Final = 84
pencil: Final = 86
pirate: Final = 88
plus: Final = 90
question_arrow: Final = 92
right_ptr: Final = 94
right_side: Final = 96
right_tee: Final = 98
rightbutton: Final = 100
rtl_logo: Final = 102
sailboat: Final = 104
sb_down_arrow: Final = 106
sb_h_double_arrow: Final = 108
sb_left_arrow: Final = 110
sb_right_arrow: Final = 112
sb_up_arrow: Final = 114
sb_v_double_arrow: Final = 116
shuttle: Final = 118
sizing: Final = 120
spider: Final = 122
spraycan: Final = 124
star: Final = 126
target: Final = 128
tcross: Final = 130
top_left_arrow: Final = 132
top_left_corner: Final = 134
top_right_corner: Final = 136
top_side: Final = 138
top_tee: Final = 140
trek: Final = 142
ul_angle: Final = 144
umbrella: Final = 146
ur_angle: Final = 148
watch: Final = 150
xterm: Final = 152



from typing import Final

DECIMAL: Final = 0
TINY: Final = 1
SHORT: Final = 2
LONG: Final = 3
FLOAT: Final = 4
DOUBLE: Final = 5
NULL: Final = 6
TIMESTAMP: Final = 7
LONGLONG: Final = 8
INT24: Final = 9
DATE: Final = 10
TIME: Final = 11
DATETIME: Final = 12
YEAR: Final = 13
NEWDATE: Final = 14
VARCHAR: Final = 15
BIT: Final = 16
JSON: Final = 245
NEWDECIMAL: Final = 246
ENUM: Final = 247
SET: Final = 248
TINY_BLOB: Final = 249
MEDIUM_BLOB: Final = 250
LONG_BLOB: Final = 251
BLOB: Final = 252
VAR_STRING: Final = 253
STRING: Final = 254
GEOMETRY: Final = 255
CHAR: Final = TINY
INTERVAL: Final = ENUM



import email.policy
from email.message import EmailMessage, Message
from email.parser import BytesParser, Parser
from typing_extensions import assert_type

p1 = Parser()
p2 = Parser(policy=email.policy.default)

assert_type(p1, Parser[Message[str, str]])
assert_type(p2, Parser[EmailMessage])

bp1 = BytesParser()
bp2 = BytesParser(policy=email.policy.default)

assert_type(bp1, BytesParser[Message[str, str]])
assert_type(bp2, BytesParser[EmailMessage])



from __future__ import annotations

from decimal import Decimal
from fractions import Fraction
from math import prod
from typing import Any, Literal, Union
from typing_extensions import assert_type


class SupportsMul:
    def __mul__(self, other: Any) -> SupportsMul:
        return SupportsMul()


class SupportsRMul:
    def __rmul__(self, other: Any) -> SupportsRMul:
        return SupportsRMul()


class SupportsMulAndRMul:
    def __mul__(self, other: Any) -> SupportsMulAndRMul:
        return SupportsMulAndRMul()

    def __rmul__(self, other: Any) -> SupportsMulAndRMul:
        return SupportsMulAndRMul()


literal_list: list[Literal[0, 1]] = [0, 1, 1]

assert_type(prod([2, 4]), int)
assert_type(prod([3, 5], start=4), int)

assert_type(prod([True, False]), int)
assert_type(prod([True, False], start=True), int)
assert_type(prod(literal_list), int)

assert_type(prod([SupportsMul(), SupportsMul()], start=SupportsMul()), SupportsMul)
assert_type(prod([SupportsMulAndRMul(), SupportsMulAndRMul()]), Union[SupportsMulAndRMul, Literal[1]])

assert_type(prod([5.6, 3.2]), Union[float, Literal[1]])
assert_type(prod([5.6, 3.2], start=3), Union[float, int])

assert_type(prod([Fraction(7, 2), Fraction(3, 5)]), Union[Fraction, Literal[1]])
assert_type(prod([Fraction(7, 2), Fraction(3, 5)], start=Fraction(1)), Fraction)
assert_type(prod([Decimal("3.14"), Decimal("2.71")]), Union[Decimal, Literal[1]])
assert_type(prod([Decimal("3.14"), Decimal("2.71")], start=Decimal("1.00")), Decimal)
assert_type(prod([complex(7, 2), complex(3, 5)]), Union[complex, Literal[1]])
assert_type(prod([complex(7, 2), complex(3, 5)], start=complex(1, 0)), complex)


# mypy and pyright infer the types differently for these, so we can't use assert_type
# Just test that no error is emitted for any of these
prod([5.6, 3.2])  # mypy: `float`; pyright: `float | Literal[0]`
prod([2.5, 5.8], start=5)  # mypy: `float`; pyright: `float | int`

# These all fail at runtime
prod([SupportsMul(), SupportsMul()])  # type: ignore
prod([SupportsRMul(), SupportsRMul()], start=SupportsRMul())  # type: ignore
prod([SupportsRMul(), SupportsRMul()])  # type: ignore

# TODO: these pass pyright with the current stubs, but mypy erroneously emits an error:
# prod([3, Fraction(7, 22), complex(8, 0), 9.83])
# prod([3, Decimal("0.98")])



from collections.abc import Generator
from typing import IO, Any, overload
from typing_extensions import Self

@overload
def reverse_iter_lines(
    file_obj: IO[bytes], blocksize: int = 4096, preseek: bool = True, encoding: None = None
) -> Generator[bytes, None, None]: ...
@overload
def reverse_iter_lines(
    file_obj: IO[str], blocksize: int = 4096, preseek: bool = True, *, encoding: str
) -> Generator[str, None, None]: ...
@overload
def reverse_iter_lines(file_obj: IO[str], blocksize: int, preseek: bool, encoding: str) -> Generator[str, None, None]: ...

class JSONLIterator:
    ignore_errors: bool
    def __init__(
        self, file_obj: IO[str], ignore_errors: bool = False, reverse: bool = False, rel_seek: float | None = None
    ) -> None: ...
    @property
    def cur_byte_pos(self) -> int: ...
    def __iter__(self) -> Self: ...
    def next(self) -> Any: ...
    __next__ = next

__all__ = ["JSONLIterator", "reverse_iter_lines"]



from _typeshed import StrPath

def augpath(
    path: StrPath,
    suffix: str = "",
    prefix: str = "",
    ext: str | None = None,
    base: str | None = None,
    dpath: str | None = None,
    multidot: bool = False,
) -> str: ...
def shrinkuser(path: StrPath, home: str = "~") -> str: ...
def expandpath(path: StrPath) -> str: ...

__all__ = ["augpath", "shrinkuser", "expandpath"]



from logging import Logger

from authlib.oauth2.rfc6749 import BaseGrant, TokenEndpointMixin

log: Logger

class ResourceOwnerPasswordCredentialsGrant(BaseGrant, TokenEndpointMixin):
    GRANT_TYPE: str
    def validate_token_request(self) -> None: ...
    def create_token_response(self): ...
    def authenticate_user(self, username, password) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, final

from google._upb._message import Descriptor, FileDescriptor, MessageMeta
from google.protobuf import message

DESCRIPTOR: FileDescriptor

@final
class HealthCheckRequest(message.Message, metaclass=MessageMeta):
    SERVICE_FIELD_NUMBER: ClassVar[int]
    service: str
    def __init__(self, service: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class HealthCheckResponse(message.Message, metaclass=MessageMeta):
    ServingStatus: Incomplete
    UNKNOWN: Incomplete
    SERVING: Incomplete
    NOT_SERVING: Incomplete
    SERVICE_UNKNOWN: Incomplete
    STATUS_FIELD_NUMBER: ClassVar[int]
    status: Incomplete
    def __init__(self, status: Incomplete | str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor



from __future__ import annotations

import platform
from typing_extensions import assert_type

# platform.uname_result emulates a 6 field named tuple, but on 3.9+ the processor
# field is lazily evaluated, which results in it being a little funky.
uname = platform.uname()
myuname = platform.uname_result("Darwin", "local", "22.5.0", "Darwin Kernel Version 22.5.0", "arm64")

assert_type(uname, platform.uname_result)
assert_type(myuname, platform.uname_result)

assert_type(uname[5], str)
assert_type(myuname[5], str)



from networkx.utils.backends import _dispatchable

__all__ = ["karate_club_graph", "davis_southern_women_graph", "florentine_families_graph", "les_miserables_graph"]

@_dispatchable
def karate_club_graph(): ...
@_dispatchable
def davis_southern_women_graph(): ...
@_dispatchable
def florentine_families_graph(): ...
@_dispatchable
def les_miserables_graph(): ...



from logging import Logger
from typing import ClassVar

log: Logger

class SDKConfig:
    XRAY_ENABLED_KEY: ClassVar[str]
    DISABLED_ENTITY_NAME: ClassVar[str]
    @classmethod
    def sdk_enabled(cls) -> bool: ...
    @classmethod
    def set_sdk_enabled(cls, value: bool | None) -> None: ...
